from typing import List
from pyVmomi import sms, vim, vmodl
from datetime import datetime


class DeviceId(vmodl.DynamicData): ...


class FailoverParam(vmodl.DynamicData):
    @property
    def isPlanned(self) -> bool: ...
    @isPlanned.setter
    def isPlanned(self, value: bool):
        self._isPlanned = value
    @property
    def checkOnly(self) -> bool: ...
    @checkOnly.setter
    def checkOnly(self, value: bool):
        self._checkOnly = value
    @property
    def replicationGroupsToFailover(self) -> List[FailoverParam.ReplicationGroupData]: ...
    @replicationGroupsToFailover.setter
    def replicationGroupsToFailover(self, value: List[FailoverParam.ReplicationGroupData]):
        self._replicationGroupsToFailover = value
    @property
    def policyAssociations(self) -> List[FailoverParam.PolicyAssociation]: ...
    @policyAssociations.setter
    def policyAssociations(self, value: List[FailoverParam.PolicyAssociation]):
        self._policyAssociations = value


    class PolicyAssociation(vmodl.DynamicData):
        @property
        def id(self) -> DeviceId: ...
        @id.setter
        def id(self, value: DeviceId):
            self._id = value
        @property
        def policyId(self) -> str: ...
        @policyId.setter
        def policyId(self, value: str):
            self._policyId = value
        @property
        def datastore(self) -> vim.Datastore: ...
        @datastore.setter
        def datastore(self, value: vim.Datastore):
            self._datastore = value


    class ReplicationGroupData(vmodl.DynamicData):
        @property
        def groupId(self) -> vim.vm.replication.ReplicationGroupId: ...
        @groupId.setter
        def groupId(self, value: vim.vm.replication.ReplicationGroupId):
            self._groupId = value
        @property
        def pitId(self) -> PointInTimeReplicaId: ...
        @pitId.setter
        def pitId(self, value: PointInTimeReplicaId):
            self._pitId = value


class FailoverSuccessResult(GroupOperationResult):
    @property
    def newState(self) -> str: ...
    @newState.setter
    def newState(self, value: str):
        self._newState = value
    @property
    def pitId(self) -> PointInTimeReplicaId: ...
    @pitId.setter
    def pitId(self, value: PointInTimeReplicaId):
        self._pitId = value
    @property
    def pitIdBeforeFailover(self) -> PointInTimeReplicaId: ...
    @pitIdBeforeFailover.setter
    def pitIdBeforeFailover(self, value: PointInTimeReplicaId):
        self._pitIdBeforeFailover = value
    @property
    def recoveredDeviceInfo(self) -> List[FailoverSuccessResult.RecoveredDevice]: ...
    @recoveredDeviceInfo.setter
    def recoveredDeviceInfo(self, value: List[FailoverSuccessResult.RecoveredDevice]):
        self._recoveredDeviceInfo = value
    @property
    def timeStamp(self) -> datetime: ...
    @timeStamp.setter
    def timeStamp(self, value: datetime):
        self._timeStamp = value


    class RecoveredDevice(vmodl.DynamicData):
        @property
        def targetDeviceId(self) -> ReplicaId: ...
        @targetDeviceId.setter
        def targetDeviceId(self, value: ReplicaId):
            self._targetDeviceId = value
        @property
        def recoveredDeviceId(self) -> DeviceId: ...
        @recoveredDeviceId.setter
        def recoveredDeviceId(self, value: DeviceId):
            self._recoveredDeviceId = value
        @property
        def sourceDeviceId(self) -> DeviceId: ...
        @sourceDeviceId.setter
        def sourceDeviceId(self, value: DeviceId):
            self._sourceDeviceId = value
        @property
        def info(self) -> List[str]: ...
        @info.setter
        def info(self, value: List[str]):
            self._info = value
        @property
        def datastore(self) -> vim.Datastore: ...
        @datastore.setter
        def datastore(self, value: vim.Datastore):
            self._datastore = value
        @property
        def recoveredDiskInfo(self) -> List[FailoverSuccessResult.RecoveredDiskInfo]: ...
        @recoveredDiskInfo.setter
        def recoveredDiskInfo(self, value: List[FailoverSuccessResult.RecoveredDiskInfo]):
            self._recoveredDiskInfo = value
        @property
        def error(self) -> vmodl.MethodFault: ...
        @error.setter
        def error(self, value: vmodl.MethodFault):
            self._error = value
        @property
        def warnings(self) -> List[vmodl.MethodFault]: ...
        @warnings.setter
        def warnings(self, value: List[vmodl.MethodFault]):
            self._warnings = value


    class RecoveredDiskInfo(vmodl.DynamicData):
        @property
        def deviceKey(self) -> int: ...
        @deviceKey.setter
        def deviceKey(self, value: int):
            self._deviceKey = value
        @property
        def dsUrl(self) -> str: ...
        @dsUrl.setter
        def dsUrl(self, value: str):
            self._dsUrl = value
        @property
        def diskPath(self) -> str: ...
        @diskPath.setter
        def diskPath(self, value: str):
            self._diskPath = value


class FaultDomainInfo(vim.vm.replication.FaultDomainId):
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, value: str):
        self._name = value
    @property
    def description(self) -> str: ...
    @description.setter
    def description(self, value: str):
        self._description = value
    @property
    def storageArrayId(self) -> str: ...
    @storageArrayId.setter
    def storageArrayId(self, value: str):
        self._storageArrayId = value
    @property
    def children(self) -> List[vim.vm.replication.FaultDomainId]: ...
    @children.setter
    def children(self, value: List[vim.vm.replication.FaultDomainId]):
        self._children = value
    @property
    def provider(self) -> sms.provider.Provider: ...
    @provider.setter
    def provider(self, value: sms.provider.Provider):
        self._provider = value


class GroupErrorResult(GroupOperationResult):
    @property
    def error(self) -> List[vmodl.MethodFault]: ...
    @error.setter
    def error(self, value: List[vmodl.MethodFault]):
        self._error = value


class GroupInfo(vmodl.DynamicData):
    @property
    def groupId(self) -> vim.vm.replication.ReplicationGroupId: ...
    @groupId.setter
    def groupId(self, value: vim.vm.replication.ReplicationGroupId):
        self._groupId = value


class GroupOperationResult(vmodl.DynamicData):
    @property
    def groupId(self) -> vim.vm.replication.ReplicationGroupId: ...
    @groupId.setter
    def groupId(self, value: vim.vm.replication.ReplicationGroupId):
        self._groupId = value
    @property
    def warning(self) -> List[vmodl.MethodFault]: ...
    @warning.setter
    def warning(self, value: List[vmodl.MethodFault]):
        self._warning = value


class PointInTimeReplicaId(vmodl.DynamicData):
    @property
    def id(self) -> str: ...
    @id.setter
    def id(self, value: str):
        self._id = value


class PromoteParam(vmodl.DynamicData):
    @property
    def isPlanned(self) -> bool: ...
    @isPlanned.setter
    def isPlanned(self, value: bool):
        self._isPlanned = value
    @property
    def replicationGroupsToPromote(self) -> List[vim.vm.replication.ReplicationGroupId]: ...
    @replicationGroupsToPromote.setter
    def replicationGroupsToPromote(self, value: List[vim.vm.replication.ReplicationGroupId]):
        self._replicationGroupsToPromote = value


class QueryPointInTimeReplicaParam(vmodl.DynamicData):
    @property
    def replicaTimeQueryParam(self) -> QueryPointInTimeReplicaParam.ReplicaQueryIntervalParam: ...
    @replicaTimeQueryParam.setter
    def replicaTimeQueryParam(self, value: QueryPointInTimeReplicaParam.ReplicaQueryIntervalParam):
        self._replicaTimeQueryParam = value
    @property
    def pitName(self) -> str: ...
    @pitName.setter
    def pitName(self, value: str):
        self._pitName = value
    @property
    def tags(self) -> List[str]: ...
    @tags.setter
    def tags(self, value: List[str]):
        self._tags = value
    @property
    def preferDetails(self) -> bool: ...
    @preferDetails.setter
    def preferDetails(self, value: bool):
        self._preferDetails = value


    class ReplicaQueryIntervalParam(vmodl.DynamicData):
        @property
        def fromDate(self) -> datetime: ...
        @fromDate.setter
        def fromDate(self, value: datetime):
            self._fromDate = value
        @property
        def toDate(self) -> datetime: ...
        @toDate.setter
        def toDate(self, value: datetime):
            self._toDate = value
        @property
        def number(self) -> int: ...
        @number.setter
        def number(self, value: int):
            self._number = value


class QueryPointInTimeReplicaSuccessResult(GroupOperationResult):
    @property
    def replicaInfo(self) -> List[QueryPointInTimeReplicaSuccessResult.PointInTimeReplicaInfo]: ...
    @replicaInfo.setter
    def replicaInfo(self, value: List[QueryPointInTimeReplicaSuccessResult.PointInTimeReplicaInfo]):
        self._replicaInfo = value


    class PointInTimeReplicaInfo(vmodl.DynamicData):
        @property
        def id(self) -> PointInTimeReplicaId: ...
        @id.setter
        def id(self, value: PointInTimeReplicaId):
            self._id = value
        @property
        def pitName(self) -> str: ...
        @pitName.setter
        def pitName(self, value: str):
            self._pitName = value
        @property
        def timeStamp(self) -> datetime: ...
        @timeStamp.setter
        def timeStamp(self, value: datetime):
            self._timeStamp = value
        @property
        def tags(self) -> List[str]: ...
        @tags.setter
        def tags(self, value: List[str]):
            self._tags = value


class QueryPointInTimeReplicaSummaryResult(GroupOperationResult):
    @property
    def intervalResults(self) -> List[QueryPointInTimeReplicaSummaryResult.ReplicaIntervalQueryResult]: ...
    @intervalResults.setter
    def intervalResults(self, value: List[QueryPointInTimeReplicaSummaryResult.ReplicaIntervalQueryResult]):
        self._intervalResults = value


    class ReplicaIntervalQueryResult(vmodl.DynamicData):
        @property
        def fromDate(self) -> datetime: ...
        @fromDate.setter
        def fromDate(self, value: datetime):
            self._fromDate = value
        @property
        def toDate(self) -> datetime: ...
        @toDate.setter
        def toDate(self, value: datetime):
            self._toDate = value
        @property
        def number(self) -> int: ...
        @number.setter
        def number(self, value: int):
            self._number = value


class QueryReplicationGroupSuccessResult(GroupOperationResult):
    @property
    def rgInfo(self) -> GroupInfo: ...
    @rgInfo.setter
    def rgInfo(self, value: GroupInfo):
        self._rgInfo = value


class QueryReplicationPeerResult(vmodl.DynamicData):
    @property
    def sourceDomain(self) -> vim.vm.replication.FaultDomainId: ...
    @sourceDomain.setter
    def sourceDomain(self, value: vim.vm.replication.FaultDomainId):
        self._sourceDomain = value
    @property
    def targetDomain(self) -> List[vim.vm.replication.FaultDomainId]: ...
    @targetDomain.setter
    def targetDomain(self, value: List[vim.vm.replication.FaultDomainId]):
        self._targetDomain = value
    @property
    def error(self) -> List[vmodl.MethodFault]: ...
    @error.setter
    def error(self, value: List[vmodl.MethodFault]):
        self._error = value
    @property
    def warning(self) -> List[vmodl.MethodFault]: ...
    @warning.setter
    def warning(self, value: List[vmodl.MethodFault]):
        self._warning = value


class RecoveredTargetGroupMemberInfo(TargetGroupMemberInfo):
    @property
    def recoveredDeviceId(self) -> DeviceId: ...
    @recoveredDeviceId.setter
    def recoveredDeviceId(self, value: DeviceId):
        self._recoveredDeviceId = value


class ReplicaId(vmodl.DynamicData):
    @property
    def id(self) -> str: ...
    @id.setter
    def id(self, value: str):
        self._id = value


class ReverseReplicationSuccessResult(GroupOperationResult):
    @property
    def newGroupId(self) -> vim.vm.replication.DeviceGroupId: ...
    @newGroupId.setter
    def newGroupId(self, value: vim.vm.replication.DeviceGroupId):
        self._newGroupId = value


class SourceGroupInfo(GroupInfo):
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, value: str):
        self._name = value
    @property
    def description(self) -> str: ...
    @description.setter
    def description(self, value: str):
        self._description = value
    @property
    def state(self) -> str: ...
    @state.setter
    def state(self, value: str):
        self._state = value
    @property
    def replica(self) -> List[SourceGroupInfo.ReplicationTargetInfo]: ...
    @replica.setter
    def replica(self, value: List[SourceGroupInfo.ReplicationTargetInfo]):
        self._replica = value
    @property
    def memberInfo(self) -> List[SourceGroupMemberInfo]: ...
    @memberInfo.setter
    def memberInfo(self, value: List[SourceGroupMemberInfo]):
        self._memberInfo = value


    class ReplicationTargetInfo(vmodl.DynamicData):
        @property
        def targetGroupId(self) -> vim.vm.replication.ReplicationGroupId: ...
        @targetGroupId.setter
        def targetGroupId(self, value: vim.vm.replication.ReplicationGroupId):
            self._targetGroupId = value
        @property
        def replicationAgreementDescription(self) -> str: ...
        @replicationAgreementDescription.setter
        def replicationAgreementDescription(self, value: str):
            self._replicationAgreementDescription = value


class SourceGroupMemberInfo(vmodl.DynamicData):
    @property
    def deviceId(self) -> DeviceId: ...
    @deviceId.setter
    def deviceId(self, value: DeviceId):
        self._deviceId = value
    @property
    def targetId(self) -> List[SourceGroupMemberInfo.TargetDeviceId]: ...
    @targetId.setter
    def targetId(self, value: List[SourceGroupMemberInfo.TargetDeviceId]):
        self._targetId = value


    class TargetDeviceId(vmodl.DynamicData):
        @property
        def domainId(self) -> vim.vm.replication.FaultDomainId: ...
        @domainId.setter
        def domainId(self, value: vim.vm.replication.FaultDomainId):
            self._domainId = value
        @property
        def deviceId(self) -> ReplicaId: ...
        @deviceId.setter
        def deviceId(self, value: ReplicaId):
            self._deviceId = value


class SyncReplicationGroupSuccessResult(GroupOperationResult):
    @property
    def timeStamp(self) -> datetime: ...
    @timeStamp.setter
    def timeStamp(self, value: datetime):
        self._timeStamp = value
    @property
    def pitId(self) -> PointInTimeReplicaId: ...
    @pitId.setter
    def pitId(self, value: PointInTimeReplicaId):
        self._pitId = value
    @property
    def pitName(self) -> str: ...
    @pitName.setter
    def pitName(self, value: str):
        self._pitName = value


class TargetGroupInfo(GroupInfo):
    @property
    def sourceInfo(self) -> TargetGroupInfo.TargetToSourceInfo: ...
    @sourceInfo.setter
    def sourceInfo(self, value: TargetGroupInfo.TargetToSourceInfo):
        self._sourceInfo = value
    @property
    def state(self) -> str: ...
    @state.setter
    def state(self, value: str):
        self._state = value
    @property
    def devices(self) -> List[TargetGroupMemberInfo]: ...
    @devices.setter
    def devices(self, value: List[TargetGroupMemberInfo]):
        self._devices = value
    @property
    def isPromoteCapable(self) -> bool: ...
    @isPromoteCapable.setter
    def isPromoteCapable(self, value: bool):
        self._isPromoteCapable = value
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, value: str):
        self._name = value


    class TargetToSourceInfo(vmodl.DynamicData):
        @property
        def sourceGroupId(self) -> vim.vm.replication.ReplicationGroupId: ...
        @sourceGroupId.setter
        def sourceGroupId(self, value: vim.vm.replication.ReplicationGroupId):
            self._sourceGroupId = value
        @property
        def replicationAgreementDescription(self) -> str: ...
        @replicationAgreementDescription.setter
        def replicationAgreementDescription(self, value: str):
            self._replicationAgreementDescription = value


class TargetGroupMemberInfo(vmodl.DynamicData):
    @property
    def replicaId(self) -> ReplicaId: ...
    @replicaId.setter
    def replicaId(self, value: ReplicaId):
        self._replicaId = value
    @property
    def sourceId(self) -> DeviceId: ...
    @sourceId.setter
    def sourceId(self, value: DeviceId):
        self._sourceId = value
    @property
    def targetDatastore(self) -> vim.Datastore: ...
    @targetDatastore.setter
    def targetDatastore(self, value: vim.Datastore):
        self._targetDatastore = value


class TestFailoverParam(FailoverParam): ...


class VVolId(DeviceId):
    @property
    def id(self) -> str: ...
    @id.setter
    def id(self, value: str):
        self._id = value


class VirtualDiskId(DeviceId):
    @property
    def diskId(self) -> str: ...
    @diskId.setter
    def diskId(self, value: str):
        self._diskId = value


class VirtualDiskKey(DeviceId):
    @property
    def vmInstanceUUID(self) -> str: ...
    @vmInstanceUUID.setter
    def vmInstanceUUID(self, value: str):
        self._vmInstanceUUID = value
    @property
    def deviceKey(self) -> int: ...
    @deviceKey.setter
    def deviceKey(self, value: int):
        self._deviceKey = value


class VirtualDiskMoId(DeviceId):
    @property
    def vcUuid(self) -> str: ...
    @vcUuid.setter
    def vcUuid(self, value: str):
        self._vcUuid = value
    @property
    def vmMoid(self) -> str: ...
    @vmMoid.setter
    def vmMoid(self, value: str):
        self._vmMoid = value
    @property
    def diskKey(self) -> str: ...
    @diskKey.setter
    def diskKey(self, value: str):
        self._diskKey = value


class VirtualMachineFilePath(VirtualMachineId):
    @property
    def vcUuid(self) -> str: ...
    @vcUuid.setter
    def vcUuid(self, value: str):
        self._vcUuid = value
    @property
    def dsUrl(self) -> str: ...
    @dsUrl.setter
    def dsUrl(self, value: str):
        self._dsUrl = value
    @property
    def vmxPath(self) -> str: ...
    @vmxPath.setter
    def vmxPath(self, value: str):
        self._vmxPath = value


class VirtualMachineId(DeviceId): ...


class VirtualMachineMoId(VirtualMachineId):
    @property
    def vcUuid(self) -> str: ...
    @vcUuid.setter
    def vcUuid(self, value: str):
        self._vcUuid = value
    @property
    def vmMoid(self) -> str: ...
    @vmMoid.setter
    def vmMoid(self, value: str):
        self._vmMoid = value


class VirtualMachineUUID(VirtualMachineId):
    @property
    def vmInstanceUUID(self) -> str: ...
    @vmInstanceUUID.setter
    def vmInstanceUUID(self, value: str):
        self._vmInstanceUUID = value


# **********************************************************
# Copyright (c) 2005-2023 VMware, Inc.
# **********************************************************

import sys
import importlib

if sys.version_info < (2, 7, 9):
    sys.stderr.write("pyVmomi requires Python 2.7.9 or newer")
    sys.exit(1)

_initialized = False


# Definition precedes pyVmomi modules imports to escape circular
# dependency error if modules import _assert_not_initialized()
def _assert_not_initialized():
    if _initialized:
        raise RuntimeError("pyVmomi is already initialized!")


try:
    settings = importlib.import_module('.pyVmomiSettings', 'pyVmomi')
except ImportError:
    settings = None

# set default settings
_allowGetSet = getattr(settings, 'allowGetSet', True)
_allowCapitalizedNames = \
    getattr(settings, 'allowCapitalizedNames', True)
_binaryIsBytearray = \
    getattr(settings, 'binaryIsBytearray', False)
_legacyThumbprintException = \
    getattr(settings, 'legacyThumbprintException', False)


from . import VmomiSupport  # noqa: E402
from . import Feature  # noqa: E402
from ._typeinfos import load_typeinfos  # noqa: E402
load_typeinfos()

# All data object types and fault types have DynamicData as an ancestor
# As well load it proactively.
# Note: This should be done before importing SoapAdapter as it uses
# some fault types
VmomiSupport.GetVmodlType("vmodl.DynamicData")

from .SoapAdapter import (  # noqa: E402,F401
    SessionOrientedStub, SoapStubAdapter, StubAdapterBase)
if _legacyThumbprintException:
    from .Security import ThumbprintMismatchException  # noqa: F401

types = VmomiSupport.types

# This will allow files to use Create** functions
# directly from pyVmomi
CreateEnumType = VmomiSupport.CreateEnumType
CreateDataType = VmomiSupport.CreateDataType
CreateManagedType = VmomiSupport.CreateManagedType

# For all the top level names, creating a LazyModule object
# in the global namespace of pyVmomi. Files can just import the
# top level namespace and we will figure out what to load and when
# Examples:
# ALLOWED: from pyVmomi import vim
# NOT ALLOWED: from pyVmomi import vim.host
_globals = globals()
for name in VmomiSupport._topLevelNames:
    upperCaseName = VmomiSupport.Capitalize(name)
    obj = VmomiSupport.LazyModule(name)
    _globals[name] = obj
    if _allowCapitalizedNames:
        _globals[upperCaseName] = obj
    if not hasattr(VmomiSupport.types, name):
        setattr(VmomiSupport.types, name, obj)
        if _allowCapitalizedNames:
            setattr(VmomiSupport.types, upperCaseName, obj)
del _globals


def init():
    _assert_not_initialized()
    Feature._init()
    global _initialized
    _initialized = True



# VMware vSphere Python SDK
# Copyright (c) 2009-2021 VMware, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from setuptools import setup


def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname)) as fn:
        return fn.read()

with open('requirements.txt') as f:
    required = f.read().splitlines()

with open('test-requirements.txt') as f:
    required_for_tests = f.read().splitlines()

setup(
    name='pyvmomi',
    version='8.0.2.0',
    description='VMware vSphere Python SDK',
    # NOTE: pypi prefers the use of RST to render docs
    long_description=read('README.rst'),
    url='https://github.com/vmware/pyvmomi',
    author='VMware, Inc.',
    author_email='jhu@vmware.com',
    packages=['pyVmomi', 'pyVim'],
    package_data={"pyVmomi": ["py.typed", "*.pyi", "**/*.pyi"]},
    install_requires=required,
    license='License :: OSI Approved :: Apache Software License',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: Apache Software License',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Developers',
        'Environment :: No Input/Output (Daemon)',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Distributed Computing',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Operating System :: MacOS',
    ],
    keywords='pyvmomi, vsphere, vmware, esx',
    platforms=['Windows', 'Linux', 'Solaris', 'Mac OS-X', 'Unix'],
    python_requires='>=2.7.9, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
    test_suite='tests',
    tests_require=required_for_tests,
    extras_require={
        'sso': [
            'pyOpenSSL',
            'lxml',
            'pywin32; sys_platform == "win32"'
        ]
    },
    zip_safe=True
)



from typing import List, Literal
from enum import Enum
from pyVmomi import vim, vmodl
from pyVmomi.VmomiSupport import byte


class AdapterMapping(vmodl.DynamicData):
    @property
    def macAddress(self) -> str: ...
    @macAddress.setter
    def macAddress(self, value: str):
        self._macAddress = value
    @property
    def adapter(self) -> IPSettings: ...
    @adapter.setter
    def adapter(self, value: IPSettings):
        self._adapter = value


class AutoIpV6Generator(IpV6Generator): ...


class CloudinitPrep(IdentitySettings):
    @property
    def metadata(self) -> str: ...
    @metadata.setter
    def metadata(self, value: str):
        self._metadata = value
    @property
    def userdata(self) -> str: ...
    @userdata.setter
    def userdata(self, value: str):
        self._userdata = value


class CustomIpGenerator(IpGenerator):
    @property
    def argument(self) -> str: ...
    @argument.setter
    def argument(self, value: str):
        self._argument = value


class CustomIpV6Generator(IpV6Generator):
    @property
    def argument(self) -> str: ...
    @argument.setter
    def argument(self, value: str):
        self._argument = value


class CustomNameGenerator(NameGenerator):
    @property
    def argument(self) -> str: ...
    @argument.setter
    def argument(self, value: str):
        self._argument = value


class DhcpIpGenerator(IpGenerator): ...


class DhcpIpV6Generator(IpV6Generator): ...


class FixedIp(IpGenerator):
    @property
    def ipAddress(self) -> str: ...
    @ipAddress.setter
    def ipAddress(self, value: str):
        self._ipAddress = value


class FixedIpV6(IpV6Generator):
    @property
    def ipAddress(self) -> str: ...
    @ipAddress.setter
    def ipAddress(self, value: str):
        self._ipAddress = value
    @property
    def subnetMask(self) -> int: ...
    @subnetMask.setter
    def subnetMask(self, value: int):
        self._subnetMask = value


class FixedName(NameGenerator):
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, value: str):
        self._name = value


class GlobalIPSettings(vmodl.DynamicData):
    @property
    def dnsSuffixList(self) -> List[str]: ...
    @dnsSuffixList.setter
    def dnsSuffixList(self, value: List[str]):
        self._dnsSuffixList = value
    @property
    def dnsServerList(self) -> List[str]: ...
    @dnsServerList.setter
    def dnsServerList(self, value: List[str]):
        self._dnsServerList = value


class GuiRunOnce(vmodl.DynamicData):
    @property
    def commandList(self) -> List[str]: ...
    @commandList.setter
    def commandList(self, value: List[str]):
        self._commandList = value


class GuiUnattended(vmodl.DynamicData):
    @property
    def password(self) -> Password: ...
    @password.setter
    def password(self, value: Password):
        self._password = value
    @property
    def timeZone(self) -> int: ...
    @timeZone.setter
    def timeZone(self, value: int):
        self._timeZone = value
    @property
    def autoLogon(self) -> bool: ...
    @autoLogon.setter
    def autoLogon(self, value: bool):
        self._autoLogon = value
    @property
    def autoLogonCount(self) -> int: ...
    @autoLogonCount.setter
    def autoLogonCount(self, value: int):
        self._autoLogonCount = value


class IPSettings(vmodl.DynamicData):
    @property
    def ip(self) -> IpGenerator: ...
    @ip.setter
    def ip(self, value: IpGenerator):
        self._ip = value
    @property
    def subnetMask(self) -> str: ...
    @subnetMask.setter
    def subnetMask(self, value: str):
        self._subnetMask = value
    @property
    def gateway(self) -> List[str]: ...
    @gateway.setter
    def gateway(self, value: List[str]):
        self._gateway = value
    @property
    def ipV6Spec(self) -> IPSettings.IpV6AddressSpec: ...
    @ipV6Spec.setter
    def ipV6Spec(self, value: IPSettings.IpV6AddressSpec):
        self._ipV6Spec = value
    @property
    def dnsServerList(self) -> List[str]: ...
    @dnsServerList.setter
    def dnsServerList(self, value: List[str]):
        self._dnsServerList = value
    @property
    def dnsDomain(self) -> str: ...
    @dnsDomain.setter
    def dnsDomain(self, value: str):
        self._dnsDomain = value
    @property
    def primaryWINS(self) -> str: ...
    @primaryWINS.setter
    def primaryWINS(self, value: str):
        self._primaryWINS = value
    @property
    def secondaryWINS(self) -> str: ...
    @secondaryWINS.setter
    def secondaryWINS(self, value: str):
        self._secondaryWINS = value
    @property
    def netBIOS(self) -> IPSettings.NetBIOSMode | Literal['enableNetBIOSViaDhcp', 'enableNetBIOS', 'disableNetBIOS']: ...
    @netBIOS.setter
    def netBIOS(self, value: IPSettings.NetBIOSMode | Literal['enableNetBIOSViaDhcp', 'enableNetBIOS', 'disableNetBIOS']):
        self._netBIOS = value


    class IpV6AddressSpec(vmodl.DynamicData):
        @property
        def ip(self) -> List[IpV6Generator]: ...
        @ip.setter
        def ip(self, value: List[IpV6Generator]):
            self._ip = value
        @property
        def gateway(self) -> List[str]: ...
        @gateway.setter
        def gateway(self, value: List[str]):
            self._gateway = value


    class NetBIOSMode(Enum):
        enableNetBIOSViaDhcp = "enableNetBIOSViaDhcp"
        enableNetBIOS = "enableNetBIOS"
        disableNetBIOS = "disableNetBIOS"


class Identification(vmodl.DynamicData):
    @property
    def joinWorkgroup(self) -> str: ...
    @joinWorkgroup.setter
    def joinWorkgroup(self, value: str):
        self._joinWorkgroup = value
    @property
    def joinDomain(self) -> str: ...
    @joinDomain.setter
    def joinDomain(self, value: str):
        self._joinDomain = value
    @property
    def domainAdmin(self) -> str: ...
    @domainAdmin.setter
    def domainAdmin(self, value: str):
        self._domainAdmin = value
    @property
    def domainAdminPassword(self) -> Password: ...
    @domainAdminPassword.setter
    def domainAdminPassword(self, value: Password):
        self._domainAdminPassword = value
    @property
    def domainOU(self) -> str: ...
    @domainOU.setter
    def domainOU(self, value: str):
        self._domainOU = value


class IdentitySettings(vmodl.DynamicData): ...


class IpGenerator(vmodl.DynamicData): ...


class IpV6Generator(vmodl.DynamicData): ...


class LicenseFilePrintData(vmodl.DynamicData):
    @property
    def autoMode(self) -> LicenseFilePrintData.AutoMode | Literal['perServer', 'perSeat']: ...
    @autoMode.setter
    def autoMode(self, value: LicenseFilePrintData.AutoMode | Literal['perServer', 'perSeat']):
        self._autoMode = value
    @property
    def autoUsers(self) -> int: ...
    @autoUsers.setter
    def autoUsers(self, value: int):
        self._autoUsers = value


    class AutoMode(Enum):
        perServer = "perServer"
        perSeat = "perSeat"


class LinuxOptions(Options): ...


class LinuxPrep(IdentitySettings):
    @property
    def hostName(self) -> NameGenerator: ...
    @hostName.setter
    def hostName(self, value: NameGenerator):
        self._hostName = value
    @property
    def domain(self) -> str: ...
    @domain.setter
    def domain(self, value: str):
        self._domain = value
    @property
    def timeZone(self) -> str: ...
    @timeZone.setter
    def timeZone(self, value: str):
        self._timeZone = value
    @property
    def hwClockUTC(self) -> bool: ...
    @hwClockUTC.setter
    def hwClockUTC(self, value: bool):
        self._hwClockUTC = value
    @property
    def scriptText(self) -> str: ...
    @scriptText.setter
    def scriptText(self, value: str):
        self._scriptText = value


class NameGenerator(vmodl.DynamicData): ...


class Options(vmodl.DynamicData): ...


class Password(vmodl.DynamicData):
    @property
    def value(self) -> str: ...
    @value.setter
    def value(self, value: str):
        self._value = value
    @property
    def plainText(self) -> bool: ...
    @plainText.setter
    def plainText(self, value: bool):
        self._plainText = value


class PrefixNameGenerator(NameGenerator):
    @property
    def base(self) -> str: ...
    @base.setter
    def base(self, value: str):
        self._base = value


class Specification(vmodl.DynamicData):
    @property
    def options(self) -> Options: ...
    @options.setter
    def options(self, value: Options):
        self._options = value
    @property
    def identity(self) -> IdentitySettings: ...
    @identity.setter
    def identity(self, value: IdentitySettings):
        self._identity = value
    @property
    def globalIPSettings(self) -> GlobalIPSettings: ...
    @globalIPSettings.setter
    def globalIPSettings(self, value: GlobalIPSettings):
        self._globalIPSettings = value
    @property
    def nicSettingMap(self) -> List[AdapterMapping]: ...
    @nicSettingMap.setter
    def nicSettingMap(self, value: List[AdapterMapping]):
        self._nicSettingMap = value
    @property
    def encryptionKey(self) -> List[byte]: ...
    @encryptionKey.setter
    def encryptionKey(self, value: List[byte]):
        self._encryptionKey = value


class StatelessIpV6Generator(IpV6Generator): ...


class Sysprep(IdentitySettings):
    @property
    def guiUnattended(self) -> GuiUnattended: ...
    @guiUnattended.setter
    def guiUnattended(self, value: GuiUnattended):
        self._guiUnattended = value
    @property
    def userData(self) -> UserData: ...
    @userData.setter
    def userData(self, value: UserData):
        self._userData = value
    @property
    def guiRunOnce(self) -> GuiRunOnce: ...
    @guiRunOnce.setter
    def guiRunOnce(self, value: GuiRunOnce):
        self._guiRunOnce = value
    @property
    def identification(self) -> Identification: ...
    @identification.setter
    def identification(self, value: Identification):
        self._identification = value
    @property
    def licenseFilePrintData(self) -> LicenseFilePrintData: ...
    @licenseFilePrintData.setter
    def licenseFilePrintData(self, value: LicenseFilePrintData):
        self._licenseFilePrintData = value


class SysprepText(IdentitySettings):
    @property
    def value(self) -> str: ...
    @value.setter
    def value(self, value: str):
        self._value = value


class UnknownIpGenerator(IpGenerator): ...


class UnknownIpV6Generator(IpV6Generator): ...


class UnknownNameGenerator(NameGenerator): ...


class UserData(vmodl.DynamicData):
    @property
    def fullName(self) -> str: ...
    @fullName.setter
    def fullName(self, value: str):
        self._fullName = value
    @property
    def orgName(self) -> str: ...
    @orgName.setter
    def orgName(self, value: str):
        self._orgName = value
    @property
    def computerName(self) -> NameGenerator: ...
    @computerName.setter
    def computerName(self, value: NameGenerator):
        self._computerName = value
    @property
    def productId(self) -> str: ...
    @productId.setter
    def productId(self, value: str):
        self._productId = value


class VirtualMachineNameGenerator(NameGenerator): ...


class WinOptions(Options):
    @property
    def changeSID(self) -> bool: ...
    @changeSID.setter
    def changeSID(self, value: bool):
        self._changeSID = value
    @property
    def deleteAccounts(self) -> bool: ...
    @deleteAccounts.setter
    def deleteAccounts(self, value: bool):
        self._deleteAccounts = value
    @property
    def reboot(self) -> WinOptions.SysprepRebootOption | Literal['reboot', 'noreboot', 'shutdown']: ...
    @reboot.setter
    def reboot(self, value: WinOptions.SysprepRebootOption | Literal['reboot', 'noreboot', 'shutdown']):
        self._reboot = value


    class SysprepRebootOption(Enum):
        reboot = "reboot"
        noreboot = "noreboot"
        shutdown = "shutdown"


from typing import List
from pyVmomi import pbm, vmodl


class DescriptiveValue(vmodl.DynamicData):
    @property
    def description(self) -> pbm.ExtendedElementDescription: ...
    @description.setter
    def description(self, value: pbm.ExtendedElementDescription):
        self._description = value
    @property
    def value(self) -> object: ...
    @value.setter
    def value(self, value: object):
        self._value = value


class DiscreteSet(vmodl.DynamicData):
    @property
    def values(self) -> List[object]: ...
    @values.setter
    def values(self, value: List[object]):
        self._values = value


class Range(vmodl.DynamicData):
    @property
    def min(self) -> object: ...
    @min.setter
    def min(self, value: object):
        self._min = value
    @property
    def max(self) -> object: ...
    @max.setter
    def max(self, value: object):
        self._max = value


class TimeSpan(vmodl.DynamicData):
    @property
    def value(self) -> int: ...
    @value.setter
    def value(self, value: int):
        self._value = value
    @property
    def unit(self) -> str: ...
    @unit.setter
    def unit(self, value: str):
        self._unit = value


from typing import List
from enum import Enum
from pyVmomi import vim, vmodl
from datetime import datetime
from pyVmomi.VmomiSupport import ManagedObject, NoneType, binary, long


class AliasManager(ManagedObject):
    def AddAlias(self, vm: vim.VirtualMachine, auth: GuestAuthentication, username: str, mapCert: bool, base64Cert: str, aliasInfo: AliasManager.GuestAuthAliasInfo) -> NoneType: ...
    def RemoveAlias(self, vm: vim.VirtualMachine, auth: GuestAuthentication, username: str, base64Cert: str, subject: AliasManager.GuestAuthSubject) -> NoneType: ...
    def RemoveAliasByCert(self, vm: vim.VirtualMachine, auth: GuestAuthentication, username: str, base64Cert: str) -> NoneType: ...
    def ListAliases(self, vm: vim.VirtualMachine, auth: GuestAuthentication, username: str) -> List[AliasManager.GuestAliases]: ...
    def ListMappedAliases(self, vm: vim.VirtualMachine, auth: GuestAuthentication) -> List[AliasManager.GuestMappedAliases]: ...


    class GuestAliases(vmodl.DynamicData):
        @property
        def base64Cert(self) -> str: ...
        @base64Cert.setter
        def base64Cert(self, value: str):
            self._base64Cert = value
        @property
        def aliases(self) -> List[AliasManager.GuestAuthAliasInfo]: ...
        @aliases.setter
        def aliases(self, value: List[AliasManager.GuestAuthAliasInfo]):
            self._aliases = value


    class GuestAuthAliasInfo(vmodl.DynamicData):
        @property
        def subject(self) -> AliasManager.GuestAuthSubject: ...
        @subject.setter
        def subject(self, value: AliasManager.GuestAuthSubject):
            self._subject = value
        @property
        def comment(self) -> str: ...
        @comment.setter
        def comment(self, value: str):
            self._comment = value


    class GuestAuthAnySubject(AliasManager.GuestAuthSubject): ...


    class GuestAuthNamedSubject(AliasManager.GuestAuthSubject):
        @property
        def name(self) -> str: ...
        @name.setter
        def name(self, value: str):
            self._name = value


    class GuestAuthSubject(vmodl.DynamicData): ...


    class GuestMappedAliases(vmodl.DynamicData):
        @property
        def base64Cert(self) -> str: ...
        @base64Cert.setter
        def base64Cert(self, value: str):
            self._base64Cert = value
        @property
        def username(self) -> str: ...
        @username.setter
        def username(self, value: str):
            self._username = value
        @property
        def subjects(self) -> List[AliasManager.GuestAuthSubject]: ...
        @subjects.setter
        def subjects(self, value: List[AliasManager.GuestAuthSubject]):
            self._subjects = value


class AuthManager(ManagedObject):
    def ValidateCredentials(self, vm: vim.VirtualMachine, auth: GuestAuthentication) -> NoneType: ...
    def AcquireCredentials(self, vm: vim.VirtualMachine, requestedAuth: GuestAuthentication, sessionID: long) -> GuestAuthentication: ...
    def ReleaseCredentials(self, vm: vim.VirtualMachine, auth: GuestAuthentication) -> NoneType: ...


class FileManager(ManagedObject):
    def MakeDirectory(self, vm: vim.VirtualMachine, auth: GuestAuthentication, directoryPath: str, createParentDirectories: bool) -> NoneType: ...
    def DeleteFile(self, vm: vim.VirtualMachine, auth: GuestAuthentication, filePath: str) -> NoneType: ...
    def DeleteDirectory(self, vm: vim.VirtualMachine, auth: GuestAuthentication, directoryPath: str, recursive: bool) -> NoneType: ...
    def MoveDirectory(self, vm: vim.VirtualMachine, auth: GuestAuthentication, srcDirectoryPath: str, dstDirectoryPath: str) -> NoneType: ...
    def MoveFile(self, vm: vim.VirtualMachine, auth: GuestAuthentication, srcFilePath: str, dstFilePath: str, overwrite: bool) -> NoneType: ...
    def CreateTemporaryFile(self, vm: vim.VirtualMachine, auth: GuestAuthentication, prefix: str, suffix: str, directoryPath: str) -> str: ...
    def CreateTemporaryDirectory(self, vm: vim.VirtualMachine, auth: GuestAuthentication, prefix: str, suffix: str, directoryPath: str) -> str: ...
    def ListFiles(self, vm: vim.VirtualMachine, auth: GuestAuthentication, filePath: str, index: int, maxResults: int, matchPattern: str) -> FileManager.ListFileInfo: ...
    def ChangeFileAttributes(self, vm: vim.VirtualMachine, auth: GuestAuthentication, guestFilePath: str, fileAttributes: FileManager.FileAttributes) -> NoneType: ...
    def InitiateFileTransferFromGuest(self, vm: vim.VirtualMachine, auth: GuestAuthentication, guestFilePath: str) -> FileManager.FileTransferInformation: ...
    def InitiateFileTransferToGuest(self, vm: vim.VirtualMachine, auth: GuestAuthentication, guestFilePath: str, fileAttributes: FileManager.FileAttributes, fileSize: long, overwrite: bool) -> str: ...


    class FileAttributes(vmodl.DynamicData):
        @property
        def modificationTime(self) -> datetime: ...
        @modificationTime.setter
        def modificationTime(self, value: datetime):
            self._modificationTime = value
        @property
        def accessTime(self) -> datetime: ...
        @accessTime.setter
        def accessTime(self, value: datetime):
            self._accessTime = value
        @property
        def symlinkTarget(self) -> str: ...
        @symlinkTarget.setter
        def symlinkTarget(self, value: str):
            self._symlinkTarget = value


    class FileInfo(vmodl.DynamicData):
        @property
        def path(self) -> str: ...
        @path.setter
        def path(self, value: str):
            self._path = value
        @property
        def type(self) -> str: ...
        @type.setter
        def type(self, value: str):
            self._type = value
        @property
        def size(self) -> long: ...
        @size.setter
        def size(self, value: long):
            self._size = value
        @property
        def attributes(self) -> FileManager.FileAttributes: ...
        @attributes.setter
        def attributes(self, value: FileManager.FileAttributes):
            self._attributes = value


        class FileType(Enum):
            file = "file"
            directory = "directory"
            symlink = "symlink"


    class FileTransferInformation(vmodl.DynamicData):
        @property
        def attributes(self) -> FileManager.FileAttributes: ...
        @attributes.setter
        def attributes(self, value: FileManager.FileAttributes):
            self._attributes = value
        @property
        def size(self) -> long: ...
        @size.setter
        def size(self, value: long):
            self._size = value
        @property
        def url(self) -> str: ...
        @url.setter
        def url(self, value: str):
            self._url = value


    class ListFileInfo(vmodl.DynamicData):
        @property
        def files(self) -> List[FileManager.FileInfo]: ...
        @files.setter
        def files(self, value: List[FileManager.FileInfo]):
            self._files = value
        @property
        def remaining(self) -> int: ...
        @remaining.setter
        def remaining(self, value: int):
            self._remaining = value


    class PosixFileAttributes(FileManager.FileAttributes):
        @property
        def ownerId(self) -> int: ...
        @ownerId.setter
        def ownerId(self, value: int):
            self._ownerId = value
        @property
        def groupId(self) -> int: ...
        @groupId.setter
        def groupId(self, value: int):
            self._groupId = value
        @property
        def permissions(self) -> long: ...
        @permissions.setter
        def permissions(self, value: long):
            self._permissions = value


    class WindowsFileAttributes(FileManager.FileAttributes):
        @property
        def hidden(self) -> bool: ...
        @hidden.setter
        def hidden(self, value: bool):
            self._hidden = value
        @property
        def readOnly(self) -> bool: ...
        @readOnly.setter
        def readOnly(self, value: bool):
            self._readOnly = value
        @property
        def createTime(self) -> datetime: ...
        @createTime.setter
        def createTime(self, value: datetime):
            self._createTime = value


class GuestOperationsManager(ManagedObject):
    @property
    def authManager(self) -> AuthManager: ...
    @property
    def fileManager(self) -> FileManager: ...
    @property
    def processManager(self) -> ProcessManager: ...
    @property
    def guestWindowsRegistryManager(self) -> WindowsRegistryManager: ...
    @property
    def aliasManager(self) -> AliasManager: ...


class ProcessManager(ManagedObject):
    def StartProgram(self, vm: vim.VirtualMachine, auth: GuestAuthentication, spec: ProcessManager.ProgramSpec) -> long: ...
    def ListProcesses(self, vm: vim.VirtualMachine, auth: GuestAuthentication, pids: List[long]) -> List[ProcessManager.ProcessInfo]: ...
    def TerminateProcess(self, vm: vim.VirtualMachine, auth: GuestAuthentication, pid: long) -> NoneType: ...
    def ReadEnvironmentVariable(self, vm: vim.VirtualMachine, auth: GuestAuthentication, names: List[str]) -> List[str]: ...


    class ProcessInfo(vmodl.DynamicData):
        @property
        def name(self) -> str: ...
        @name.setter
        def name(self, value: str):
            self._name = value
        @property
        def pid(self) -> long: ...
        @pid.setter
        def pid(self, value: long):
            self._pid = value
        @property
        def owner(self) -> str: ...
        @owner.setter
        def owner(self, value: str):
            self._owner = value
        @property
        def cmdLine(self) -> str: ...
        @cmdLine.setter
        def cmdLine(self, value: str):
            self._cmdLine = value
        @property
        def startTime(self) -> datetime: ...
        @startTime.setter
        def startTime(self, value: datetime):
            self._startTime = value
        @property
        def endTime(self) -> datetime: ...
        @endTime.setter
        def endTime(self, value: datetime):
            self._endTime = value
        @property
        def exitCode(self) -> int: ...
        @exitCode.setter
        def exitCode(self, value: int):
            self._exitCode = value


    class ProgramSpec(vmodl.DynamicData):
        @property
        def programPath(self) -> str: ...
        @programPath.setter
        def programPath(self, value: str):
            self._programPath = value
        @property
        def arguments(self) -> str: ...
        @arguments.setter
        def arguments(self, value: str):
            self._arguments = value
        @property
        def workingDirectory(self) -> str: ...
        @workingDirectory.setter
        def workingDirectory(self, value: str):
            self._workingDirectory = value
        @property
        def envVariables(self) -> List[str]: ...
        @envVariables.setter
        def envVariables(self, value: List[str]):
            self._envVariables = value


    class WindowsProgramSpec(ProcessManager.ProgramSpec):
        @property
        def startMinimized(self) -> bool: ...
        @startMinimized.setter
        def startMinimized(self, value: bool):
            self._startMinimized = value


class WindowsRegistryManager(ManagedObject):
    def CreateRegistryKey(self, vm: vim.VirtualMachine, auth: GuestAuthentication, keyName: WindowsRegistryManager.RegistryKeyName, isVolatile: bool, classType: str) -> NoneType: ...
    def ListRegistryKeys(self, vm: vim.VirtualMachine, auth: GuestAuthentication, keyName: WindowsRegistryManager.RegistryKeyName, recursive: bool, matchPattern: str) -> List[WindowsRegistryManager.RegistryKeyRecord]: ...
    def DeleteRegistryKey(self, vm: vim.VirtualMachine, auth: GuestAuthentication, keyName: WindowsRegistryManager.RegistryKeyName, recursive: bool) -> NoneType: ...
    def SetRegistryValue(self, vm: vim.VirtualMachine, auth: GuestAuthentication, value: WindowsRegistryManager.RegistryValue) -> NoneType: ...
    def ListRegistryValues(self, vm: vim.VirtualMachine, auth: GuestAuthentication, keyName: WindowsRegistryManager.RegistryKeyName, expandStrings: bool, matchPattern: str) -> List[WindowsRegistryManager.RegistryValue]: ...
    def DeleteRegistryValue(self, vm: vim.VirtualMachine, auth: GuestAuthentication, valueName: WindowsRegistryManager.RegistryValueName) -> NoneType: ...


    class RegistryKey(vmodl.DynamicData):
        @property
        def keyName(self) -> WindowsRegistryManager.RegistryKeyName: ...
        @keyName.setter
        def keyName(self, value: WindowsRegistryManager.RegistryKeyName):
            self._keyName = value
        @property
        def classType(self) -> str: ...
        @classType.setter
        def classType(self, value: str):
            self._classType = value
        @property
        def lastWritten(self) -> datetime: ...
        @lastWritten.setter
        def lastWritten(self, value: datetime):
            self._lastWritten = value


    class RegistryKeyName(vmodl.DynamicData):
        @property
        def registryPath(self) -> str: ...
        @registryPath.setter
        def registryPath(self, value: str):
            self._registryPath = value
        @property
        def wowBitness(self) -> str: ...
        @wowBitness.setter
        def wowBitness(self, value: str):
            self._wowBitness = value


        class RegistryKeyWowBitness(Enum):
            WOWNative = "WOWNative"
            WOW32 = "WOW32"
            WOW64 = "WOW64"


    class RegistryKeyRecord(vmodl.DynamicData):
        @property
        def key(self) -> WindowsRegistryManager.RegistryKey: ...
        @key.setter
        def key(self, value: WindowsRegistryManager.RegistryKey):
            self._key = value
        @property
        def fault(self) -> vmodl.MethodFault: ...
        @fault.setter
        def fault(self, value: vmodl.MethodFault):
            self._fault = value


    class RegistryValue(vmodl.DynamicData):
        @property
        def name(self) -> WindowsRegistryManager.RegistryValueName: ...
        @name.setter
        def name(self, value: WindowsRegistryManager.RegistryValueName):
            self._name = value
        @property
        def data(self) -> WindowsRegistryManager.RegistryValueData: ...
        @data.setter
        def data(self, value: WindowsRegistryManager.RegistryValueData):
            self._data = value


    class RegistryValueBinary(WindowsRegistryManager.RegistryValueData):
        @property
        def value(self) -> binary: ...
        @value.setter
        def value(self, value: binary):
            self._value = value


    class RegistryValueData(vmodl.DynamicData): ...


    class RegistryValueDword(WindowsRegistryManager.RegistryValueData):
        @property
        def value(self) -> int: ...
        @value.setter
        def value(self, value: int):
            self._value = value


    class RegistryValueExpandString(WindowsRegistryManager.RegistryValueData):
        @property
        def value(self) -> str: ...
        @value.setter
        def value(self, value: str):
            self._value = value


    class RegistryValueMultiString(WindowsRegistryManager.RegistryValueData):
        @property
        def value(self) -> List[str]: ...
        @value.setter
        def value(self, value: List[str]):
            self._value = value


    class RegistryValueName(vmodl.DynamicData):
        @property
        def keyName(self) -> WindowsRegistryManager.RegistryKeyName: ...
        @keyName.setter
        def keyName(self, value: WindowsRegistryManager.RegistryKeyName):
            self._keyName = value
        @property
        def name(self) -> str: ...
        @name.setter
        def name(self, value: str):
            self._name = value


    class RegistryValueQword(WindowsRegistryManager.RegistryValueData):
        @property
        def value(self) -> long: ...
        @value.setter
        def value(self, value: long):
            self._value = value


    class RegistryValueString(WindowsRegistryManager.RegistryValueData):
        @property
        def value(self) -> str: ...
        @value.setter
        def value(self, value: str):
            self._value = value


class GuestAuthentication(vmodl.DynamicData):
    @property
    def interactiveSession(self) -> bool: ...
    @interactiveSession.setter
    def interactiveSession(self, value: bool):
        self._interactiveSession = value


class NamePasswordAuthentication(GuestAuthentication):
    @property
    def username(self) -> str: ...
    @username.setter
    def username(self, value: str):
        self._username = value
    @property
    def password(self) -> str: ...
    @password.setter
    def password(self, value: str):
        self._password = value


class SAMLTokenAuthentication(GuestAuthentication):
    @property
    def token(self) -> str: ...
    @token.setter
    def token(self, value: str):
        self._token = value
    @property
    def username(self) -> str: ...
    @username.setter
    def username(self, value: str):
        self._username = value


class SSPIAuthentication(GuestAuthentication):
    @property
    def sspiToken(self) -> str: ...
    @sspiToken.setter
    def sspiToken(self, value: str):
        self._sspiToken = value


class TicketedSessionAuthentication(GuestAuthentication):
    @property
    def ticket(self) -> str: ...
    @ticket.setter
    def ticket(self, value: str):
        self._ticket = value


from typing import List, Literal
from enum import Enum
from pyVmomi import vim, vmodl
from datetime import datetime
from pyVmomi.VmomiSupport import ManagedObject, NoneType, PropertyPath


class Alarm(vim.ExtensibleManagedObject):
    @property
    def info(self) -> AlarmInfo: ...
    def Remove(self) -> NoneType: ...
    def Reconfigure(self, spec: AlarmSpec) -> NoneType: ...


class AlarmManager(ManagedObject):
    @property
    def defaultExpression(self) -> List[AlarmExpression]: ...
    @property
    def description(self) -> AlarmDescription: ...
    def Create(self, entity: vim.ManagedEntity, spec: AlarmSpec) -> Alarm: ...
    def GetAlarm(self, entity: vim.ManagedEntity) -> List[Alarm]: ...
    def GetAlarmActionsEnabled(self, entity: vim.ManagedEntity) -> bool: ...
    def SetAlarmActionsEnabled(self, entity: vim.ManagedEntity, enabled: bool) -> NoneType: ...
    def GetAlarmState(self, entity: vim.ManagedEntity) -> List[AlarmState]: ...
    def AcknowledgeAlarm(self, alarm: Alarm, entity: vim.ManagedEntity) -> NoneType: ...
    def ClearTriggeredAlarms(self, filter: AlarmFilterSpec) -> NoneType: ...
    def DisableAlarm(self, alarm: Alarm, entity: vim.ManagedEntity) -> NoneType: ...
    def EnableAlarm(self, alarm: Alarm, entity: vim.ManagedEntity) -> NoneType: ...


class AlarmAction(vmodl.DynamicData): ...


class AlarmDescription(vmodl.DynamicData):
    @property
    def expr(self) -> List[vim.TypeDescription]: ...
    @expr.setter
    def expr(self, value: List[vim.TypeDescription]):
        self._expr = value
    @property
    def stateOperator(self) -> List[vim.ElementDescription]: ...
    @stateOperator.setter
    def stateOperator(self, value: List[vim.ElementDescription]):
        self._stateOperator = value
    @property
    def metricOperator(self) -> List[vim.ElementDescription]: ...
    @metricOperator.setter
    def metricOperator(self, value: List[vim.ElementDescription]):
        self._metricOperator = value
    @property
    def hostSystemConnectionState(self) -> List[vim.ElementDescription]: ...
    @hostSystemConnectionState.setter
    def hostSystemConnectionState(self, value: List[vim.ElementDescription]):
        self._hostSystemConnectionState = value
    @property
    def virtualMachinePowerState(self) -> List[vim.ElementDescription]: ...
    @virtualMachinePowerState.setter
    def virtualMachinePowerState(self, value: List[vim.ElementDescription]):
        self._virtualMachinePowerState = value
    @property
    def datastoreConnectionState(self) -> List[vim.ElementDescription]: ...
    @datastoreConnectionState.setter
    def datastoreConnectionState(self, value: List[vim.ElementDescription]):
        self._datastoreConnectionState = value
    @property
    def hostSystemPowerState(self) -> List[vim.ElementDescription]: ...
    @hostSystemPowerState.setter
    def hostSystemPowerState(self, value: List[vim.ElementDescription]):
        self._hostSystemPowerState = value
    @property
    def virtualMachineGuestHeartbeatStatus(self) -> List[vim.ElementDescription]: ...
    @virtualMachineGuestHeartbeatStatus.setter
    def virtualMachineGuestHeartbeatStatus(self, value: List[vim.ElementDescription]):
        self._virtualMachineGuestHeartbeatStatus = value
    @property
    def entityStatus(self) -> List[vim.ElementDescription]: ...
    @entityStatus.setter
    def entityStatus(self, value: List[vim.ElementDescription]):
        self._entityStatus = value
    @property
    def action(self) -> List[vim.TypeDescription]: ...
    @action.setter
    def action(self, value: List[vim.TypeDescription]):
        self._action = value


class AlarmExpression(vmodl.DynamicData): ...


class AlarmFilterSpec(vmodl.DynamicData):
    @property
    def status(self) -> List[vim.ManagedEntity.Status]: ...
    @status.setter
    def status(self, value: List[vim.ManagedEntity.Status]):
        self._status = value
    @property
    def typeEntity(self) -> str: ...
    @typeEntity.setter
    def typeEntity(self, value: str):
        self._typeEntity = value
    @property
    def typeTrigger(self) -> str: ...
    @typeTrigger.setter
    def typeTrigger(self, value: str):
        self._typeTrigger = value


    class AlarmTypeByEntity(Enum):
        entityTypeAll = "entityTypeAll"
        entityTypeHost = "entityTypeHost"
        entityTypeVm = "entityTypeVm"


    class AlarmTypeByTrigger(Enum):
        triggerTypeAll = "triggerTypeAll"
        triggerTypeEvent = "triggerTypeEvent"
        triggerTypeMetric = "triggerTypeMetric"


class AlarmInfo(AlarmSpec):
    @property
    def key(self) -> str: ...
    @key.setter
    def key(self, value: str):
        self._key = value
    @property
    def alarm(self) -> Alarm: ...
    @alarm.setter
    def alarm(self, value: Alarm):
        self._alarm = value
    @property
    def entity(self) -> vim.ManagedEntity: ...
    @entity.setter
    def entity(self, value: vim.ManagedEntity):
        self._entity = value
    @property
    def lastModifiedTime(self) -> datetime: ...
    @lastModifiedTime.setter
    def lastModifiedTime(self, value: datetime):
        self._lastModifiedTime = value
    @property
    def lastModifiedUser(self) -> str: ...
    @lastModifiedUser.setter
    def lastModifiedUser(self, value: str):
        self._lastModifiedUser = value
    @property
    def creationEventId(self) -> int: ...
    @creationEventId.setter
    def creationEventId(self, value: int):
        self._creationEventId = value


class AlarmSetting(vmodl.DynamicData):
    @property
    def toleranceRange(self) -> int: ...
    @toleranceRange.setter
    def toleranceRange(self, value: int):
        self._toleranceRange = value
    @property
    def reportingFrequency(self) -> int: ...
    @reportingFrequency.setter
    def reportingFrequency(self, value: int):
        self._reportingFrequency = value


class AlarmSpec(vmodl.DynamicData):
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, value: str):
        self._name = value
    @property
    def systemName(self) -> str: ...
    @systemName.setter
    def systemName(self, value: str):
        self._systemName = value
    @property
    def description(self) -> str: ...
    @description.setter
    def description(self, value: str):
        self._description = value
    @property
    def enabled(self) -> bool: ...
    @enabled.setter
    def enabled(self, value: bool):
        self._enabled = value
    @property
    def expression(self) -> AlarmExpression: ...
    @expression.setter
    def expression(self, value: AlarmExpression):
        self._expression = value
    @property
    def action(self) -> AlarmAction: ...
    @action.setter
    def action(self, value: AlarmAction):
        self._action = value
    @property
    def actionFrequency(self) -> int: ...
    @actionFrequency.setter
    def actionFrequency(self, value: int):
        self._actionFrequency = value
    @property
    def setting(self) -> AlarmSetting: ...
    @setting.setter
    def setting(self, value: AlarmSetting):
        self._setting = value


class AlarmState(vmodl.DynamicData):
    @property
    def key(self) -> str: ...
    @key.setter
    def key(self, value: str):
        self._key = value
    @property
    def entity(self) -> vim.ManagedEntity: ...
    @entity.setter
    def entity(self, value: vim.ManagedEntity):
        self._entity = value
    @property
    def alarm(self) -> Alarm: ...
    @alarm.setter
    def alarm(self, value: Alarm):
        self._alarm = value
    @property
    def overallStatus(self) -> vim.ManagedEntity.Status | Literal['gray', 'green', 'yellow', 'red']: ...
    @overallStatus.setter
    def overallStatus(self, value: vim.ManagedEntity.Status | Literal['gray', 'green', 'yellow', 'red']):
        self._overallStatus = value
    @property
    def time(self) -> datetime: ...
    @time.setter
    def time(self, value: datetime):
        self._time = value
    @property
    def acknowledged(self) -> bool: ...
    @acknowledged.setter
    def acknowledged(self, value: bool):
        self._acknowledged = value
    @property
    def acknowledgedByUser(self) -> str: ...
    @acknowledgedByUser.setter
    def acknowledgedByUser(self, value: str):
        self._acknowledgedByUser = value
    @property
    def acknowledgedTime(self) -> datetime: ...
    @acknowledgedTime.setter
    def acknowledgedTime(self, value: datetime):
        self._acknowledgedTime = value
    @property
    def eventKey(self) -> int: ...
    @eventKey.setter
    def eventKey(self, value: int):
        self._eventKey = value
    @property
    def disabled(self) -> bool: ...
    @disabled.setter
    def disabled(self, value: bool):
        self._disabled = value


class AlarmTriggeringAction(AlarmAction):
    @property
    def action(self) -> vim.action.Action: ...
    @action.setter
    def action(self, value: vim.action.Action):
        self._action = value
    @property
    def transitionSpecs(self) -> List[AlarmTriggeringAction.TransitionSpec]: ...
    @transitionSpecs.setter
    def transitionSpecs(self, value: List[AlarmTriggeringAction.TransitionSpec]):
        self._transitionSpecs = value
    @property
    def green2yellow(self) -> bool: ...
    @green2yellow.setter
    def green2yellow(self, value: bool):
        self._green2yellow = value
    @property
    def yellow2red(self) -> bool: ...
    @yellow2red.setter
    def yellow2red(self, value: bool):
        self._yellow2red = value
    @property
    def red2yellow(self) -> bool: ...
    @red2yellow.setter
    def red2yellow(self, value: bool):
        self._red2yellow = value
    @property
    def yellow2green(self) -> bool: ...
    @yellow2green.setter
    def yellow2green(self, value: bool):
        self._yellow2green = value


    class TransitionSpec(vmodl.DynamicData):
        @property
        def startState(self) -> vim.ManagedEntity.Status | Literal['gray', 'green', 'yellow', 'red']: ...
        @startState.setter
        def startState(self, value: vim.ManagedEntity.Status | Literal['gray', 'green', 'yellow', 'red']):
            self._startState = value
        @property
        def finalState(self) -> vim.ManagedEntity.Status | Literal['gray', 'green', 'yellow', 'red']: ...
        @finalState.setter
        def finalState(self, value: vim.ManagedEntity.Status | Literal['gray', 'green', 'yellow', 'red']):
            self._finalState = value
        @property
        def repeats(self) -> bool: ...
        @repeats.setter
        def repeats(self, value: bool):
            self._repeats = value


class AndAlarmExpression(AlarmExpression):
    @property
    def expression(self) -> List[AlarmExpression]: ...
    @expression.setter
    def expression(self, value: List[AlarmExpression]):
        self._expression = value


class EventAlarmExpression(AlarmExpression):
    @property
    def comparisons(self) -> List[EventAlarmExpression.Comparison]: ...
    @comparisons.setter
    def comparisons(self, value: List[EventAlarmExpression.Comparison]):
        self._comparisons = value
    @property
    def eventType(self) -> type: ...
    @eventType.setter
    def eventType(self, value: type):
        self._eventType = value
    @property
    def eventTypeId(self) -> str: ...
    @eventTypeId.setter
    def eventTypeId(self, value: str):
        self._eventTypeId = value
    @property
    def objectType(self) -> type: ...
    @objectType.setter
    def objectType(self, value: type):
        self._objectType = value
    @property
    def status(self) -> vim.ManagedEntity.Status | Literal['gray', 'green', 'yellow', 'red']: ...
    @status.setter
    def status(self, value: vim.ManagedEntity.Status | Literal['gray', 'green', 'yellow', 'red']):
        self._status = value


    class Comparison(vmodl.DynamicData):
        @property
        def attributeName(self) -> str: ...
        @attributeName.setter
        def attributeName(self, value: str):
            self._attributeName = value
        @property
        def operator(self) -> str: ...
        @operator.setter
        def operator(self, value: str):
            self._operator = value
        @property
        def value(self) -> str: ...
        @value.setter
        def value(self, value: str):
            self._value = value


    class ComparisonOperator(Enum):
        equals = "equals"
        notEqualTo = "notEqualTo"
        startsWith = "startsWith"
        doesNotStartWith = "doesNotStartWith"
        endsWith = "endsWith"
        doesNotEndWith = "doesNotEndWith"


class GroupAlarmAction(AlarmAction):
    @property
    def action(self) -> List[AlarmAction]: ...
    @action.setter
    def action(self, value: List[AlarmAction]):
        self._action = value


class MetricAlarmExpression(AlarmExpression):
    @property
    def operator(self) -> MetricAlarmExpression.MetricOperator | Literal['isAbove', 'isBelow']: ...
    @operator.setter
    def operator(self, value: MetricAlarmExpression.MetricOperator | Literal['isAbove', 'isBelow']):
        self._operator = value
    @property
    def type(self) -> type: ...
    @type.setter
    def type(self, value: type):
        self._type = value
    @property
    def metric(self) -> vim.PerformanceManager.MetricId: ...
    @metric.setter
    def metric(self, value: vim.PerformanceManager.MetricId):
        self._metric = value
    @property
    def yellow(self) -> int: ...
    @yellow.setter
    def yellow(self, value: int):
        self._yellow = value
    @property
    def yellowInterval(self) -> int: ...
    @yellowInterval.setter
    def yellowInterval(self, value: int):
        self._yellowInterval = value
    @property
    def red(self) -> int: ...
    @red.setter
    def red(self, value: int):
        self._red = value
    @property
    def redInterval(self) -> int: ...
    @redInterval.setter
    def redInterval(self, value: int):
        self._redInterval = value


    class MetricOperator(Enum):
        isAbove = "isAbove"
        isBelow = "isBelow"


class OrAlarmExpression(AlarmExpression):
    @property
    def expression(self) -> List[AlarmExpression]: ...
    @expression.setter
    def expression(self, value: List[AlarmExpression]):
        self._expression = value


class StateAlarmExpression(AlarmExpression):
    @property
    def operator(self) -> StateAlarmExpression.StateOperator | Literal['isEqual', 'isUnequal']: ...
    @operator.setter
    def operator(self, value: StateAlarmExpression.StateOperator | Literal['isEqual', 'isUnequal']):
        self._operator = value
    @property
    def type(self) -> type: ...
    @type.setter
    def type(self, value: type):
        self._type = value
    @property
    def statePath(self) -> PropertyPath: ...
    @statePath.setter
    def statePath(self, value: PropertyPath):
        self._statePath = value
    @property
    def yellow(self) -> str: ...
    @yellow.setter
    def yellow(self, value: str):
        self._yellow = value
    @property
    def red(self) -> str: ...
    @red.setter
    def red(self, value: str):
        self._red = value


    class StateOperator(Enum):
        isEqual = "isEqual"
        isUnequal = "isUnequal"


from typing import List
from enum import Enum
from pyVmomi import vim, vmodl
from pyVmomi.VmomiSupport import ManagedMethod, ManagedObject


class FailoverClusterConfigurator(ManagedObject):
    @property
    def disabledConfigureMethod(self) -> List[ManagedMethod]: ...
    def Prepare(self, networkSpec: FailoverClusterConfigurator.VchaClusterNetworkSpec) -> vim.Task: ...
    def Deploy(self, deploymentSpec: FailoverClusterConfigurator.VchaClusterDeploymentSpec) -> vim.Task: ...
    def Configure(self, configSpec: FailoverClusterConfigurator.VchaClusterConfigSpec) -> vim.Task: ...
    def CreatePassiveNode(self, passiveDeploymentSpec: FailoverClusterConfigurator.PassiveNodeDeploymentSpec, sourceVcSpec: FailoverClusterConfigurator.SourceNodeSpec) -> vim.Task: ...
    def CreateWitnessNode(self, witnessDeploymentSpec: FailoverClusterConfigurator.NodeDeploymentSpec, sourceVcSpec: FailoverClusterConfigurator.SourceNodeSpec) -> vim.Task: ...
    def GetConfig(self) -> FailoverClusterConfigurator.VchaClusterConfigInfo: ...
    def Destroy(self) -> vim.Task: ...


    class ClusterNetworkConfigSpec(vmodl.DynamicData):
        @property
        def networkPortGroup(self) -> vim.Network: ...
        @networkPortGroup.setter
        def networkPortGroup(self, value: vim.Network):
            self._networkPortGroup = value
        @property
        def ipSettings(self) -> vim.vm.customization.IPSettings: ...
        @ipSettings.setter
        def ipSettings(self, value: vim.vm.customization.IPSettings):
            self._ipSettings = value


    class FailoverNodeInfo(vmodl.DynamicData):
        @property
        def clusterIpSettings(self) -> vim.vm.customization.IPSettings: ...
        @clusterIpSettings.setter
        def clusterIpSettings(self, value: vim.vm.customization.IPSettings):
            self._clusterIpSettings = value
        @property
        def failoverIp(self) -> vim.vm.customization.IPSettings: ...
        @failoverIp.setter
        def failoverIp(self, value: vim.vm.customization.IPSettings):
            self._failoverIp = value
        @property
        def biosUuid(self) -> str: ...
        @biosUuid.setter
        def biosUuid(self, value: str):
            self._biosUuid = value


    class NodeDeploymentSpec(vmodl.DynamicData):
        @property
        def esxHost(self) -> vim.HostSystem: ...
        @esxHost.setter
        def esxHost(self, value: vim.HostSystem):
            self._esxHost = value
        @property
        def datastore(self) -> vim.Datastore: ...
        @datastore.setter
        def datastore(self, value: vim.Datastore):
            self._datastore = value
        @property
        def publicNetworkPortGroup(self) -> vim.Network: ...
        @publicNetworkPortGroup.setter
        def publicNetworkPortGroup(self, value: vim.Network):
            self._publicNetworkPortGroup = value
        @property
        def clusterNetworkPortGroup(self) -> vim.Network: ...
        @clusterNetworkPortGroup.setter
        def clusterNetworkPortGroup(self, value: vim.Network):
            self._clusterNetworkPortGroup = value
        @property
        def folder(self) -> vim.Folder: ...
        @folder.setter
        def folder(self, value: vim.Folder):
            self._folder = value
        @property
        def resourcePool(self) -> vim.ResourcePool: ...
        @resourcePool.setter
        def resourcePool(self, value: vim.ResourcePool):
            self._resourcePool = value
        @property
        def managementVc(self) -> vim.ServiceLocator: ...
        @managementVc.setter
        def managementVc(self, value: vim.ServiceLocator):
            self._managementVc = value
        @property
        def nodeName(self) -> str: ...
        @nodeName.setter
        def nodeName(self, value: str):
            self._nodeName = value
        @property
        def ipSettings(self) -> vim.vm.customization.IPSettings: ...
        @ipSettings.setter
        def ipSettings(self, value: vim.vm.customization.IPSettings):
            self._ipSettings = value


    class NodeNetworkSpec(vmodl.DynamicData):
        @property
        def ipSettings(self) -> vim.vm.customization.IPSettings: ...
        @ipSettings.setter
        def ipSettings(self, value: vim.vm.customization.IPSettings):
            self._ipSettings = value


    class PassiveNodeDeploymentSpec(FailoverClusterConfigurator.NodeDeploymentSpec):
        @property
        def failoverIpSettings(self) -> vim.vm.customization.IPSettings: ...
        @failoverIpSettings.setter
        def failoverIpSettings(self, value: vim.vm.customization.IPSettings):
            self._failoverIpSettings = value


    class PassiveNodeNetworkSpec(FailoverClusterConfigurator.NodeNetworkSpec):
        @property
        def failoverIpSettings(self) -> vim.vm.customization.IPSettings: ...
        @failoverIpSettings.setter
        def failoverIpSettings(self, value: vim.vm.customization.IPSettings):
            self._failoverIpSettings = value


    class SourceNodeSpec(vmodl.DynamicData):
        @property
        def managementVc(self) -> vim.ServiceLocator: ...
        @managementVc.setter
        def managementVc(self, value: vim.ServiceLocator):
            self._managementVc = value
        @property
        def activeVc(self) -> vim.VirtualMachine: ...
        @activeVc.setter
        def activeVc(self, value: vim.VirtualMachine):
            self._activeVc = value


    class VchaClusterConfigInfo(vmodl.DynamicData):
        @property
        def failoverNodeInfo1(self) -> FailoverClusterConfigurator.FailoverNodeInfo: ...
        @failoverNodeInfo1.setter
        def failoverNodeInfo1(self, value: FailoverClusterConfigurator.FailoverNodeInfo):
            self._failoverNodeInfo1 = value
        @property
        def failoverNodeInfo2(self) -> FailoverClusterConfigurator.FailoverNodeInfo: ...
        @failoverNodeInfo2.setter
        def failoverNodeInfo2(self, value: FailoverClusterConfigurator.FailoverNodeInfo):
            self._failoverNodeInfo2 = value
        @property
        def witnessNodeInfo(self) -> FailoverClusterConfigurator.WitnessNodeInfo: ...
        @witnessNodeInfo.setter
        def witnessNodeInfo(self, value: FailoverClusterConfigurator.WitnessNodeInfo):
            self._witnessNodeInfo = value
        @property
        def state(self) -> str: ...
        @state.setter
        def state(self, value: str):
            self._state = value


    class VchaClusterConfigSpec(vmodl.DynamicData):
        @property
        def passiveIp(self) -> str: ...
        @passiveIp.setter
        def passiveIp(self, value: str):
            self._passiveIp = value
        @property
        def witnessIp(self) -> str: ...
        @witnessIp.setter
        def witnessIp(self, value: str):
            self._witnessIp = value


    class VchaClusterDeploymentSpec(vmodl.DynamicData):
        @property
        def passiveDeploymentSpec(self) -> FailoverClusterConfigurator.PassiveNodeDeploymentSpec: ...
        @passiveDeploymentSpec.setter
        def passiveDeploymentSpec(self, value: FailoverClusterConfigurator.PassiveNodeDeploymentSpec):
            self._passiveDeploymentSpec = value
        @property
        def witnessDeploymentSpec(self) -> FailoverClusterConfigurator.NodeDeploymentSpec: ...
        @witnessDeploymentSpec.setter
        def witnessDeploymentSpec(self, value: FailoverClusterConfigurator.NodeDeploymentSpec):
            self._witnessDeploymentSpec = value
        @property
        def activeVcSpec(self) -> FailoverClusterConfigurator.SourceNodeSpec: ...
        @activeVcSpec.setter
        def activeVcSpec(self, value: FailoverClusterConfigurator.SourceNodeSpec):
            self._activeVcSpec = value
        @property
        def activeVcNetworkConfig(self) -> FailoverClusterConfigurator.ClusterNetworkConfigSpec: ...
        @activeVcNetworkConfig.setter
        def activeVcNetworkConfig(self, value: FailoverClusterConfigurator.ClusterNetworkConfigSpec):
            self._activeVcNetworkConfig = value


    class VchaClusterNetworkSpec(vmodl.DynamicData):
        @property
        def witnessNetworkSpec(self) -> FailoverClusterConfigurator.NodeNetworkSpec: ...
        @witnessNetworkSpec.setter
        def witnessNetworkSpec(self, value: FailoverClusterConfigurator.NodeNetworkSpec):
            self._witnessNetworkSpec = value
        @property
        def passiveNetworkSpec(self) -> FailoverClusterConfigurator.PassiveNodeNetworkSpec: ...
        @passiveNetworkSpec.setter
        def passiveNetworkSpec(self, value: FailoverClusterConfigurator.PassiveNodeNetworkSpec):
            self._passiveNetworkSpec = value


    class WitnessNodeInfo(vmodl.DynamicData):
        @property
        def ipSettings(self) -> vim.vm.customization.IPSettings: ...
        @ipSettings.setter
        def ipSettings(self, value: vim.vm.customization.IPSettings):
            self._ipSettings = value
        @property
        def biosUuid(self) -> str: ...
        @biosUuid.setter
        def biosUuid(self, value: str):
            self._biosUuid = value


    class VchaState(Enum):
        configured = "configured"
        notConfigured = "notConfigured"
        invalid = "invalid"
        prepared = "prepared"


class FailoverClusterManager(ManagedObject):
    @property
    def disabledClusterMethod(self) -> List[ManagedMethod]: ...
    def SetClusterMode(self, mode: str) -> vim.Task: ...
    def GetClusterMode(self) -> str: ...
    def GetClusterHealth(self) -> FailoverClusterManager.VchaClusterHealth: ...
    def InitiateFailover(self, planned: bool) -> vim.Task: ...


    class VchaClusterHealth(vmodl.DynamicData):
        @property
        def runtimeInfo(self) -> FailoverClusterManager.VchaClusterRuntimeInfo: ...
        @runtimeInfo.setter
        def runtimeInfo(self, value: FailoverClusterManager.VchaClusterRuntimeInfo):
            self._runtimeInfo = value
        @property
        def healthMessages(self) -> List[vmodl.LocalizableMessage]: ...
        @healthMessages.setter
        def healthMessages(self, value: List[vmodl.LocalizableMessage]):
            self._healthMessages = value
        @property
        def additionalInformation(self) -> List[vmodl.LocalizableMessage]: ...
        @additionalInformation.setter
        def additionalInformation(self, value: List[vmodl.LocalizableMessage]):
            self._additionalInformation = value


    class VchaClusterRuntimeInfo(vmodl.DynamicData):
        @property
        def clusterState(self) -> str: ...
        @clusterState.setter
        def clusterState(self, value: str):
            self._clusterState = value
        @property
        def nodeInfo(self) -> List[FailoverClusterManager.VchaNodeRuntimeInfo]: ...
        @nodeInfo.setter
        def nodeInfo(self, value: List[FailoverClusterManager.VchaNodeRuntimeInfo]):
            self._nodeInfo = value
        @property
        def clusterMode(self) -> str: ...
        @clusterMode.setter
        def clusterMode(self, value: str):
            self._clusterMode = value


    class VchaNodeRuntimeInfo(vmodl.DynamicData):
        @property
        def nodeState(self) -> str: ...
        @nodeState.setter
        def nodeState(self, value: str):
            self._nodeState = value
        @property
        def nodeRole(self) -> str: ...
        @nodeRole.setter
        def nodeRole(self, value: str):
            self._nodeRole = value
        @property
        def nodeIp(self) -> str: ...
        @nodeIp.setter
        def nodeIp(self, value: str):
            self._nodeIp = value


    class VchaClusterMode(Enum):
        enabled = "enabled"
        disabled = "disabled"
        maintenance = "maintenance"


    class VchaClusterState(Enum):
        healthy = "healthy"
        degraded = "degraded"
        isolated = "isolated"


    class VchaNodeRole(Enum):
        active = "active"
        passive = "passive"
        witness = "witness"


    class VchaNodeState(Enum):
        up = "up"
        down = "down"


from typing import List
from enum import Enum
from pyVmomi import vim, vmodl
from datetime import datetime
from pyVmomi.VmomiSupport import ManagedObject, NoneType, PropertyPath
from . import cluster as cluster
from . import host as host


class ComplianceManager(ManagedObject):
    def CheckCompliance(self, profile: List[Profile], entity: List[vim.ManagedEntity]) -> vim.Task: ...
    def QueryComplianceStatus(self, profile: List[Profile], entity: List[vim.ManagedEntity]) -> List[ComplianceResult]: ...
    def ClearComplianceStatus(self, profile: List[Profile], entity: List[vim.ManagedEntity]) -> NoneType: ...
    def QueryExpressionMetadata(self, expressionName: List[str], profile: Profile) -> List[ExpressionMetadata]: ...


class Profile(ManagedObject):
    @property
    def config(self) -> Profile.ConfigInfo: ...
    @property
    def description(self) -> Profile.Description: ...
    @property
    def name(self) -> str: ...
    @property
    def createdTime(self) -> datetime: ...
    @property
    def modifiedTime(self) -> datetime: ...
    @property
    def entity(self) -> List[vim.ManagedEntity]: ...
    @property
    def complianceStatus(self) -> str: ...
    def RetrieveDescription(self) -> Profile.Description: ...
    def Destroy(self) -> NoneType: ...
    def AssociateEntities(self, entity: List[vim.ManagedEntity]) -> NoneType: ...
    def DissociateEntities(self, entity: List[vim.ManagedEntity]) -> NoneType: ...
    def CheckCompliance(self, entity: List[vim.ManagedEntity]) -> vim.Task: ...
    def ExportProfile(self) -> str: ...


    class ConfigInfo(vmodl.DynamicData):
        @property
        def name(self) -> str: ...
        @name.setter
        def name(self, value: str):
            self._name = value
        @property
        def annotation(self) -> str: ...
        @annotation.setter
        def annotation(self, value: str):
            self._annotation = value
        @property
        def enabled(self) -> bool: ...
        @enabled.setter
        def enabled(self, value: bool):
            self._enabled = value


    class CreateSpec(vmodl.DynamicData):
        @property
        def name(self) -> str: ...
        @name.setter
        def name(self, value: str):
            self._name = value
        @property
        def annotation(self) -> str: ...
        @annotation.setter
        def annotation(self, value: str):
            self._annotation = value
        @property
        def enabled(self) -> bool: ...
        @enabled.setter
        def enabled(self, value: bool):
            self._enabled = value


    class Description(vmodl.DynamicData):
        @property
        def section(self) -> List[Profile.Description.Section]: ...
        @section.setter
        def section(self, value: List[Profile.Description.Section]):
            self._section = value


        class Section(vmodl.DynamicData):
            @property
            def description(self) -> vim.ExtendedElementDescription: ...
            @description.setter
            def description(self, value: vim.ExtendedElementDescription):
                self._description = value
            @property
            def message(self) -> List[vmodl.LocalizableMessage]: ...
            @message.setter
            def message(self, value: List[vmodl.LocalizableMessage]):
                self._message = value


    class SerializedCreateSpec(Profile.CreateSpec):
        @property
        def profileConfigString(self) -> str: ...
        @profileConfigString.setter
        def profileConfigString(self, value: str):
            self._profileConfigString = value


class ProfileManager(ManagedObject):
    @property
    def profile(self) -> List[Profile]: ...
    def CreateProfile(self, createSpec: Profile.CreateSpec) -> Profile: ...
    def QueryPolicyMetadata(self, policyName: List[str], profile: Profile) -> List[PolicyMetadata]: ...
    def FindAssociatedProfile(self, entity: vim.ManagedEntity) -> List[Profile]: ...


class ApplyProfile(vmodl.DynamicData):
    @property
    def enabled(self) -> bool: ...
    @enabled.setter
    def enabled(self, value: bool):
        self._enabled = value
    @property
    def policy(self) -> List[Policy]: ...
    @policy.setter
    def policy(self, value: List[Policy]):
        self._policy = value
    @property
    def profileTypeName(self) -> str: ...
    @profileTypeName.setter
    def profileTypeName(self, value: str):
        self._profileTypeName = value
    @property
    def profileVersion(self) -> str: ...
    @profileVersion.setter
    def profileVersion(self, value: str):
        self._profileVersion = value
    @property
    def property(self) -> List[ApplyProfileProperty]: ...
    @property.setter
    def property(self, value: List[ApplyProfileProperty]):
        self._property = value
    @property
    def favorite(self) -> bool: ...
    @favorite.setter
    def favorite(self, value: bool):
        self._favorite = value
    @property
    def toBeMerged(self) -> bool: ...
    @toBeMerged.setter
    def toBeMerged(self, value: bool):
        self._toBeMerged = value
    @property
    def toReplaceWith(self) -> bool: ...
    @toReplaceWith.setter
    def toReplaceWith(self, value: bool):
        self._toReplaceWith = value
    @property
    def toBeDeleted(self) -> bool: ...
    @toBeDeleted.setter
    def toBeDeleted(self, value: bool):
        self._toBeDeleted = value
    @property
    def copyEnableStatus(self) -> bool: ...
    @copyEnableStatus.setter
    def copyEnableStatus(self, value: bool):
        self._copyEnableStatus = value
    @property
    def hidden(self) -> bool: ...
    @hidden.setter
    def hidden(self, value: bool):
        self._hidden = value


class ApplyProfileElement(ApplyProfile):
    @property
    def key(self) -> str: ...
    @key.setter
    def key(self, value: str):
        self._key = value


class ApplyProfileProperty(vmodl.DynamicData):
    @property
    def propertyName(self) -> str: ...
    @propertyName.setter
    def propertyName(self, value: str):
        self._propertyName = value
    @property
    def array(self) -> bool: ...
    @array.setter
    def array(self, value: bool):
        self._array = value
    @property
    def profile(self) -> List[ApplyProfile]: ...
    @profile.setter
    def profile(self, value: List[ApplyProfile]):
        self._profile = value


class ComplianceLocator(vmodl.DynamicData):
    @property
    def expressionName(self) -> str: ...
    @expressionName.setter
    def expressionName(self, value: str):
        self._expressionName = value
    @property
    def applyPath(self) -> ProfilePropertyPath: ...
    @applyPath.setter
    def applyPath(self, value: ProfilePropertyPath):
        self._applyPath = value


class ComplianceProfile(vmodl.DynamicData):
    @property
    def expression(self) -> List[Expression]: ...
    @expression.setter
    def expression(self, value: List[Expression]):
        self._expression = value
    @property
    def rootExpression(self) -> str: ...
    @rootExpression.setter
    def rootExpression(self, value: str):
        self._rootExpression = value


class ComplianceResult(vmodl.DynamicData):
    @property
    def profile(self) -> Profile: ...
    @profile.setter
    def profile(self, value: Profile):
        self._profile = value
    @property
    def complianceStatus(self) -> str: ...
    @complianceStatus.setter
    def complianceStatus(self, value: str):
        self._complianceStatus = value
    @property
    def entity(self) -> vim.ManagedEntity: ...
    @entity.setter
    def entity(self, value: vim.ManagedEntity):
        self._entity = value
    @property
    def checkTime(self) -> datetime: ...
    @checkTime.setter
    def checkTime(self, value: datetime):
        self._checkTime = value
    @property
    def failure(self) -> List[ComplianceResult.ComplianceFailure]: ...
    @failure.setter
    def failure(self, value: List[ComplianceResult.ComplianceFailure]):
        self._failure = value


    class ComplianceFailure(vmodl.DynamicData):
        @property
        def failureType(self) -> str: ...
        @failureType.setter
        def failureType(self, value: str):
            self._failureType = value
        @property
        def message(self) -> vmodl.LocalizableMessage: ...
        @message.setter
        def message(self, value: vmodl.LocalizableMessage):
            self._message = value
        @property
        def expressionName(self) -> str: ...
        @expressionName.setter
        def expressionName(self, value: str):
            self._expressionName = value
        @property
        def failureValues(self) -> List[ComplianceResult.ComplianceFailure.ComplianceFailureValues]: ...
        @failureValues.setter
        def failureValues(self, value: List[ComplianceResult.ComplianceFailure.ComplianceFailureValues]):
            self._failureValues = value


        class ComplianceFailureValues(vmodl.DynamicData):
            @property
            def comparisonIdentifier(self) -> str: ...
            @comparisonIdentifier.setter
            def comparisonIdentifier(self, value: str):
                self._comparisonIdentifier = value
            @property
            def profileInstance(self) -> str: ...
            @profileInstance.setter
            def profileInstance(self, value: str):
                self._profileInstance = value
            @property
            def hostValue(self) -> object: ...
            @hostValue.setter
            def hostValue(self, value: object):
                self._hostValue = value
            @property
            def profileValue(self) -> object: ...
            @profileValue.setter
            def profileValue(self, value: object):
                self._profileValue = value


    class Status(Enum):
        compliant = "compliant"
        nonCompliant = "nonCompliant"
        unknown = "unknown"
        running = "running"


class CompositeExpression(Expression):
    @property
    def operator(self) -> str: ...
    @operator.setter
    def operator(self, value: str):
        self._operator = value
    @property
    def expressionName(self) -> List[str]: ...
    @expressionName.setter
    def expressionName(self, value: List[str]):
        self._expressionName = value


class CompositePolicyOption(PolicyOption):
    @property
    def option(self) -> List[PolicyOption]: ...
    @option.setter
    def option(self, value: List[PolicyOption]):
        self._option = value


class CompositePolicyOptionMetadata(PolicyOptionMetadata):
    @property
    def option(self) -> List[str]: ...
    @option.setter
    def option(self, value: List[str]):
        self._option = value


class DeferredPolicyOptionParameter(vmodl.DynamicData):
    @property
    def inputPath(self) -> ProfilePropertyPath: ...
    @inputPath.setter
    def inputPath(self, value: ProfilePropertyPath):
        self._inputPath = value
    @property
    def parameter(self) -> List[vmodl.KeyAnyValue]: ...
    @parameter.setter
    def parameter(self, value: List[vmodl.KeyAnyValue]):
        self._parameter = value


class Expression(vmodl.DynamicData):
    @property
    def id(self) -> str: ...
    @id.setter
    def id(self, value: str):
        self._id = value
    @property
    def displayName(self) -> str: ...
    @displayName.setter
    def displayName(self, value: str):
        self._displayName = value
    @property
    def negated(self) -> bool: ...
    @negated.setter
    def negated(self, value: bool):
        self._negated = value


class ExpressionMetadata(vmodl.DynamicData):
    @property
    def expressionId(self) -> vim.ExtendedElementDescription: ...
    @expressionId.setter
    def expressionId(self, value: vim.ExtendedElementDescription):
        self._expressionId = value
    @property
    def parameter(self) -> List[ParameterMetadata]: ...
    @parameter.setter
    def parameter(self, value: List[ParameterMetadata]):
        self._parameter = value


class ParameterMetadata(vmodl.DynamicData):
    @property
    def id(self) -> vim.ExtendedElementDescription: ...
    @id.setter
    def id(self, value: vim.ExtendedElementDescription):
        self._id = value
    @property
    def type(self) -> type: ...
    @type.setter
    def type(self, value: type):
        self._type = value
    @property
    def optional(self) -> bool: ...
    @optional.setter
    def optional(self, value: bool):
        self._optional = value
    @property
    def defaultValue(self) -> object: ...
    @defaultValue.setter
    def defaultValue(self, value: object):
        self._defaultValue = value
    @property
    def hidden(self) -> bool: ...
    @hidden.setter
    def hidden(self, value: bool):
        self._hidden = value
    @property
    def securitySensitive(self) -> bool: ...
    @securitySensitive.setter
    def securitySensitive(self, value: bool):
        self._securitySensitive = value
    @property
    def readOnly(self) -> bool: ...
    @readOnly.setter
    def readOnly(self, value: bool):
        self._readOnly = value
    @property
    def parameterRelations(self) -> List[ParameterMetadata.ParameterRelationMetadata]: ...
    @parameterRelations.setter
    def parameterRelations(self, value: List[ParameterMetadata.ParameterRelationMetadata]):
        self._parameterRelations = value


    class ParameterRelationMetadata(vmodl.DynamicData):
        @property
        def relationTypes(self) -> List[str]: ...
        @relationTypes.setter
        def relationTypes(self, value: List[str]):
            self._relationTypes = value
        @property
        def values(self) -> List[object]: ...
        @values.setter
        def values(self, value: List[object]):
            self._values = value
        @property
        def path(self) -> ProfilePropertyPath: ...
        @path.setter
        def path(self, value: ProfilePropertyPath):
            self._path = value
        @property
        def minCount(self) -> int: ...
        @minCount.setter
        def minCount(self, value: int):
            self._minCount = value
        @property
        def maxCount(self) -> int: ...
        @maxCount.setter
        def maxCount(self, value: int):
            self._maxCount = value


    class RelationType(Enum):
        dynamic_relation = "dynamic_relation"
        extensible_relation = "extensible_relation"
        localizable_relation = "localizable_relation"
        static_relation = "static_relation"
        validation_relation = "validation_relation"


class Policy(vmodl.DynamicData):
    @property
    def id(self) -> str: ...
    @id.setter
    def id(self, value: str):
        self._id = value
    @property
    def policyOption(self) -> PolicyOption: ...
    @policyOption.setter
    def policyOption(self, value: PolicyOption):
        self._policyOption = value


class PolicyMetadata(vmodl.DynamicData):
    @property
    def id(self) -> vim.ExtendedElementDescription: ...
    @id.setter
    def id(self, value: vim.ExtendedElementDescription):
        self._id = value
    @property
    def possibleOption(self) -> List[PolicyOptionMetadata]: ...
    @possibleOption.setter
    def possibleOption(self, value: List[PolicyOptionMetadata]):
        self._possibleOption = value


class PolicyOption(vmodl.DynamicData):
    @property
    def id(self) -> str: ...
    @id.setter
    def id(self, value: str):
        self._id = value
    @property
    def parameter(self) -> List[vmodl.KeyAnyValue]: ...
    @parameter.setter
    def parameter(self, value: List[vmodl.KeyAnyValue]):
        self._parameter = value


class PolicyOptionMetadata(vmodl.DynamicData):
    @property
    def id(self) -> vim.ExtendedElementDescription: ...
    @id.setter
    def id(self, value: vim.ExtendedElementDescription):
        self._id = value
    @property
    def parameter(self) -> List[ParameterMetadata]: ...
    @parameter.setter
    def parameter(self, value: List[ParameterMetadata]):
        self._parameter = value


class ProfileMetadata(vmodl.DynamicData):
    @property
    def key(self) -> type: ...
    @key.setter
    def key(self, value: type):
        self._key = value
    @property
    def profileTypeName(self) -> str: ...
    @profileTypeName.setter
    def profileTypeName(self, value: str):
        self._profileTypeName = value
    @property
    def description(self) -> vim.ExtendedDescription: ...
    @description.setter
    def description(self, value: vim.ExtendedDescription):
        self._description = value
    @property
    def sortSpec(self) -> List[ProfileMetadata.ProfileSortSpec]: ...
    @sortSpec.setter
    def sortSpec(self, value: List[ProfileMetadata.ProfileSortSpec]):
        self._sortSpec = value
    @property
    def profileCategory(self) -> str: ...
    @profileCategory.setter
    def profileCategory(self, value: str):
        self._profileCategory = value
    @property
    def profileComponent(self) -> str: ...
    @profileComponent.setter
    def profileComponent(self, value: str):
        self._profileComponent = value
    @property
    def operationMessages(self) -> List[ProfileMetadata.ProfileOperationMessage]: ...
    @operationMessages.setter
    def operationMessages(self, value: List[ProfileMetadata.ProfileOperationMessage]):
        self._operationMessages = value


    class ProfileOperationMessage(vmodl.DynamicData):
        @property
        def operationName(self) -> str: ...
        @operationName.setter
        def operationName(self, value: str):
            self._operationName = value
        @property
        def message(self) -> vmodl.LocalizableMessage: ...
        @message.setter
        def message(self, value: vmodl.LocalizableMessage):
            self._message = value


    class ProfileSortSpec(vmodl.DynamicData):
        @property
        def policyId(self) -> str: ...
        @policyId.setter
        def policyId(self, value: str):
            self._policyId = value
        @property
        def parameter(self) -> str: ...
        @parameter.setter
        def parameter(self, value: str):
            self._parameter = value


class ProfilePropertyPath(vmodl.DynamicData):
    @property
    def profilePath(self) -> PropertyPath: ...
    @profilePath.setter
    def profilePath(self, value: PropertyPath):
        self._profilePath = value
    @property
    def policyId(self) -> str: ...
    @policyId.setter
    def policyId(self, value: str):
        self._policyId = value
    @property
    def parameterId(self) -> str: ...
    @parameterId.setter
    def parameterId(self, value: str):
        self._parameterId = value
    @property
    def policyOptionId(self) -> str: ...
    @policyOptionId.setter
    def policyOptionId(self, value: str):
        self._policyOptionId = value


class ProfileStructure(vmodl.DynamicData):
    @property
    def profileTypeName(self) -> str: ...
    @profileTypeName.setter
    def profileTypeName(self, value: str):
        self._profileTypeName = value
    @property
    def child(self) -> List[ProfileStructureProperty]: ...
    @child.setter
    def child(self, value: List[ProfileStructureProperty]):
        self._child = value


class ProfileStructureProperty(vmodl.DynamicData):
    @property
    def propertyName(self) -> str: ...
    @propertyName.setter
    def propertyName(self, value: str):
        self._propertyName = value
    @property
    def array(self) -> bool: ...
    @array.setter
    def array(self, value: bool):
        self._array = value
    @property
    def element(self) -> ProfileStructure: ...
    @element.setter
    def element(self, value: ProfileStructure):
        self._element = value


class SimpleExpression(Expression):
    @property
    def expressionType(self) -> str: ...
    @expressionType.setter
    def expressionType(self, value: str):
        self._expressionType = value
    @property
    def parameter(self) -> List[vmodl.KeyAnyValue]: ...
    @parameter.setter
    def parameter(self, value: List[vmodl.KeyAnyValue]):
        self._parameter = value


class UserInputRequiredParameterMetadata(PolicyOptionMetadata):
    @property
    def userInputParameter(self) -> List[ParameterMetadata]: ...
    @userInputParameter.setter
    def userInputParameter(self, value: List[ParameterMetadata]):
        self._userInputParameter = value


from typing import List
from pyVmomi import vim, vmodl


class ExtendedProductInfo(vmodl.DynamicData):
    @property
    def companyUrl(self) -> str: ...
    @companyUrl.setter
    def companyUrl(self, value: str):
        self._companyUrl = value
    @property
    def productUrl(self) -> str: ...
    @productUrl.setter
    def productUrl(self, value: str):
        self._productUrl = value
    @property
    def managementUrl(self) -> str: ...
    @managementUrl.setter
    def managementUrl(self, value: str):
        self._managementUrl = value
    @property
    def self(self) -> vim.ManagedEntity: ...
    @self.setter
    def self(self, value: vim.ManagedEntity):
        self._self = value


class ManagedByInfo(vmodl.DynamicData):
    @property
    def extensionKey(self) -> str: ...
    @extensionKey.setter
    def extensionKey(self, value: str):
        self._extensionKey = value
    @property
    def type(self) -> str: ...
    @type.setter
    def type(self, value: str):
        self._type = value


class ManagedEntityInfo(vmodl.DynamicData):
    @property
    def type(self) -> str: ...
    @type.setter
    def type(self, value: str):
        self._type = value
    @property
    def smallIconUrl(self) -> str: ...
    @smallIconUrl.setter
    def smallIconUrl(self, value: str):
        self._smallIconUrl = value
    @property
    def iconUrl(self) -> str: ...
    @iconUrl.setter
    def iconUrl(self, value: str):
        self._iconUrl = value
    @property
    def description(self) -> str: ...
    @description.setter
    def description(self, value: str):
        self._description = value


class SolutionManagerInfo(vmodl.DynamicData):
    @property
    def tab(self) -> List[SolutionManagerInfo.TabInfo]: ...
    @tab.setter
    def tab(self, value: List[SolutionManagerInfo.TabInfo]):
        self._tab = value
    @property
    def smallIconUrl(self) -> str: ...
    @smallIconUrl.setter
    def smallIconUrl(self, value: str):
        self._smallIconUrl = value


    class TabInfo(vmodl.DynamicData):
        @property
        def label(self) -> str: ...
        @label.setter
        def label(self, value: str):
            self._label = value
        @property
        def url(self) -> str: ...
        @url.setter
        def url(self, value: str):
            self._url = value


from typing import List, Literal
from pyVmomi import vim, vmodl
from pyVmomi.VmomiSupport import ManagedObject


class CompatibilityChecker(ManagedObject):
    def CheckCompatibility(self, vm: vim.VirtualMachine, host: vim.HostSystem, pool: vim.ResourcePool, testType: List[str]) -> vim.Task: ...
    def CheckVmConfig(self, spec: vim.vm.ConfigSpec, vm: vim.VirtualMachine, host: vim.HostSystem, pool: vim.ResourcePool, testType: List[str]) -> vim.Task: ...
    def CheckPowerOn(self, vm: vim.VirtualMachine, host: vim.HostSystem, pool: vim.ResourcePool, testType: List[str]) -> vim.Task: ...


class ProvisioningChecker(ManagedObject):
    def QueryVMotionCompatibilityEx(self, vm: List[vim.VirtualMachine], host: List[vim.HostSystem]) -> vim.Task: ...
    def CheckMigrate(self, vm: vim.VirtualMachine, host: vim.HostSystem, pool: vim.ResourcePool, state: vim.VirtualMachine.PowerState | Literal['poweredOff', 'poweredOn', 'suspended'], testType: List[str]) -> vim.Task: ...
    def CheckRelocate(self, vm: vim.VirtualMachine, spec: vim.vm.RelocateSpec, testType: List[str]) -> vim.Task: ...
    def CheckClone(self, vm: vim.VirtualMachine, folder: vim.Folder, name: str, spec: vim.vm.CloneSpec, testType: List[str]) -> vim.Task: ...
    def CheckInstantClone(self, vm: vim.VirtualMachine, spec: vim.vm.InstantCloneSpec, testType: List[str]) -> vim.Task: ...


class Result(vmodl.DynamicData):
    @property
    def vm(self) -> vim.VirtualMachine: ...
    @vm.setter
    def vm(self, value: vim.VirtualMachine):
        self._vm = value
    @property
    def host(self) -> vim.HostSystem: ...
    @host.setter
    def host(self, value: vim.HostSystem):
        self._host = value
    @property
    def warning(self) -> List[vmodl.MethodFault]: ...
    @warning.setter
    def warning(self, value: List[vmodl.MethodFault]):
        self._warning = value
    @property
    def error(self) -> List[vmodl.MethodFault]: ...
    @error.setter
    def error(self, value: List[vmodl.MethodFault]):
        self._error = value


from pyVmomi import vmodl


class DeviceGroupId(vmodl.DynamicData):
    @property
    def id(self) -> str: ...
    @id.setter
    def id(self, value: str):
        self._id = value


class FaultDomainId(vmodl.DynamicData):
    @property
    def id(self) -> str: ...
    @id.setter
    def id(self, value: str):
        self._id = value


class ReplicationGroupId(vmodl.DynamicData):
    @property
    def faultDomainId(self) -> FaultDomainId: ...
    @faultDomainId.setter
    def faultDomainId(self, value: FaultDomainId):
        self._faultDomainId = value
    @property
    def deviceGroupId(self) -> DeviceGroupId: ...
    @deviceGroupId.setter
    def deviceGroupId(self, value: DeviceGroupId):
        self._deviceGroupId = value


class ReplicationSpec(vmodl.DynamicData):
    @property
    def replicationGroupId(self) -> ReplicationGroupId: ...
    @replicationGroupId.setter
    def replicationGroupId(self, value: ReplicationGroupId):
        self._replicationGroupId = value


#!/usr/bin/env python
"""
Copyright (c) 2009-2022 VMware, Inc.

This module is for ISO 8601 parsing
"""
__author__ = 'VMware, Inc'

import re
import time
from datetime import datetime, timedelta, tzinfo

import six

# Regular expression to parse a subset of ISO 8601 format
_dtExpr = re.compile(
    # XMLSchema datetime. Mandatory to have - and :
    # See: http://www.w3.org/TR/xmlschema-2/#isoformats
    # Note: python datetime cannot handle the following:
    #       - leap second, ie. 0-60 seconds (not 0-59)
    #       - BC (negative years)
    # year [-]0000..9999
    r'(?P<year>-?\d{4})'
    # month 01..12
    r'(-(?P<month>(0[1-9]|1[0-2]))'
    # day 01..31
    r'(-(?P<day>(0[1-9]|[1-2]\d|3[01])))?)?'
    # time separator 'T'
    r'(T'
    # hour 00..24
    r'(?P<hour>([01]\d|2[0-4]))'
    # minute 00..59
    r'((:(?P<minute>[0-5]\d))'
    # seconds 00..60 (leap second ok)
    r'(:(?P<second>([0-5]\d|60))'
    # microsecond. max 16 digits
    # - Should not allows trailing zeros. But python isoformat() put zeros
    #   after microseconds. Oh well, allows trailing zeros, quite harmless
    r'(\.(?P<microsecond>\d{1,16}))?)?)?'
    # UTC 'Z', or...
    r'((?P<tzutc>Z)'
    # tz [+-]00..13:0..59|14:00
    r'|((?P<tzhr>[+-](([0]\d)|(1[0-3])|(?P<tzlimit>)14))'
    r'(:(?P<tzmin>(?(tzlimit)00|([0-5]\d))))?))?'
    r')?$')

# Default date time val. Key should match the tags in _dtExpr
_dtExprKeyDefValMap = {
    'year': None,
    'month': 1,
    'day': 1,
    'hour': 0,
    'minute': 0,
    'second': 0,
    'microsecond': 0
}


class TZInfo(tzinfo):
    """ Timezone info class """

    timedelta0 = timedelta(hours=0)
    timedelta1 = timedelta(hours=1)

    def __init__(self, tzname='UTC', utcOffset=None, dst=None):
        self._tzname = tzname
        if not utcOffset:
            utcOffset = self.timedelta0
        self._utcOffset = utcOffset
        if not dst:
            dst = None
        self._dst = dst

    def utcoffset(self, dt):
        return self._utcOffset + self.dst(dt)

    def tzname(self, dt):
        return self._tzname

    def dst(self, dt):
        ret = self.timedelta0
        if self._dst:
            if self._dst[0] <= dt.replace(tzinfo=None) < self._dst[1]:
                ret = self.timedelta1
        return ret


class TZManager:
    """ Time zone manager """
    _tzInfos = {}

    @staticmethod
    def GetTZInfo(tzname='UTC', utcOffset=None, dst=None):
        """ Get / Add timezone info """
        key = (tzname, utcOffset, dst)
        tzInfo = TZManager._tzInfos.get(key)
        if not tzInfo:
            tzInfo = TZInfo(tzname, utcOffset, dst)
            TZManager._tzInfos[key] = tzInfo
        return tzInfo


def ParseISO8601(datetimeStr):
    """
    Parse ISO 8601 date time from string.
    Returns datetime if ok, None otherwise
    Note: Allows YYYY / YYYY-MM, but truncate YYYY -> YYYY-01-01,
                                              YYYY-MM -> YYYY-MM-01
    Truncate microsecond to most significant 6 digits
    """
    datetimeVal = None
    match = _dtExpr.match(datetimeStr)
    if match:
        try:
            dt = {}
            for key, defaultVal in six.iteritems(_dtExprKeyDefValMap):
                val = match.group(key)
                if val:
                    if key == 'microsecond':
                        val = val[:6] + '0' * (6 - len(val))
                    dt[key] = int(val)
                elif defaultVal:
                    dt[key] = defaultVal

            # Orig. XMLSchema don't allow all zeros year. But newer draft is ok
            # if dt['year'] == 0:
            #   # Year cannot be all zeros
            #   raise Exception('Year cannot be all zeros')

            # 24 is a special case.
            # It is actually represented as next day 00:00
            delta = None
            if dt.get('hour', 0) == 24:
                # Must be 24:00:00.0
                if (dt.get('minute', 0) == 0 and dt.get('second', 0) == 0
                        and dt.get('microsecond', 0) == 0):
                    dt['hour'] = 23
                    delta = timedelta(hours=1)
                else:
                    return None

            # Set tzinfo
            # TODO: dst
            tzInfo = None
            val = match.group('tzutc')
            if val:
                tzInfo = TZManager.GetTZInfo()
            else:
                val = match.group('tzhr')
                if val:
                    # tz hours offset
                    tzhr = int(val)
                    utcsign = val[0]

                    # tz minutes offset
                    tzmin = 0
                    val = match.group('tzmin')
                    if val:
                        tzmin = tzhr >= 0 and int(val) or -int(val)

                    # Better tzname (map UTC +-00:00 to UTC)
                    tzname = 'UTC'
                    if tzhr != 0 or tzmin != 0:
                        tzname += ' %s%02d:%02d' % (utcsign, abs(tzhr),
                                                    abs(tzmin))

                    tzInfo = TZManager.GetTZInfo(
                        tzname=tzname,
                        utcOffset=timedelta(hours=tzhr, minutes=tzmin))
            if tzInfo:
                dt['tzinfo'] = tzInfo

            datetimeVal = datetime(**dt)
            if delta:
                datetimeVal += delta
        except Exception:
            pass
    return datetimeVal


def GetUtcOffset():
    try:
        return time.localtime().tm_gmtoff
    except AttributeError:
        useAltZone = time.daylight and time.localtime().tm_isdst
        return -(time.altzone if useAltZone else time.timezone)


def ISO8601Format(dt):
    """
    Python datetime isoformat() has the following problems:
    - leave trailing 0 at the end of microseconds (violates XMLSchema rule)
    - tz print +00:00 instead of Z
    - Missing timezone offset for datetime without tzinfo
    """
    isoStr = dt.strftime('%Y-%m-%dT%H:%M:%S')
    if dt.microsecond:
        isoStr += ('.%06d' % dt.microsecond).rstrip('0')
    if dt.tzinfo:
        tz = dt.strftime('%z')
    else:
        utcOffset_minutes = GetUtcOffset() / 60
        tz = "%+.2d%.2d" % (utcOffset_minutes / 60,
                            (abs(utcOffset_minutes) % 60))
    if tz == '+0000':
        return isoStr + 'Z'
    elif tz:
        return isoStr + tz[:3] + ':' + tz[3:]
    else:
        # Local offset is unknown
        return isoStr + '-00:00'


# Testing
if __name__ == '__main__':
    # Valid entries
    for testStr in [
            '1971',  # 1971-01-01
            '1971-11',  # 1971-11-01
            '1971-11-02',
            '1971-11-02T23',
            '1971-11-02T23Z',
            '1971-11-02T23:04',
            '1971-11-02T23:04Z',
            '1971-11-02T23:04:15',
            '1971-11-02T23:04:15Z',
            '1971-11-02T23:04:15.1',
            '1971-11-02T23:04:15.01',
            '1971-11-02T23:04:15.023456',
            '1971-11-02T23:04:15.103456Z',
            '1971-11-02T23:04:15.123456+11',
            '1971-11-02T23:04:15.123456-11',
            '1971-11-02T23:04:15.123456+11:30',
            '1971-11-02T23:04:15.123456-11:30',
            '1971-11-02T23:04:15.123456+00:00',  # Same as Z
            '1971-11-02T23:04:15.123456-00:00',  # Same as Z
            '1971-01-02T23:04:15+14',
            '1971-01-02T23:04:15+14:00',
            '1971-01-02T23:04:15-14',
            '1971-01-02T23:04:15-14:00',

            # Valid: Truncate microsec to 6 digits
            '1971-01-02T23:04:15.123456891+11',
            '1971-01-02T24',  # 24 is valid. It should represent the 00:00 the
            # next day
            '1971-01-02T24:00',
            '1971-01-02T24:00:00',
            '1971-01-02T24:00:00.0',

            # Should NOT be valid but python isoformat adding trailing zeros
            '1971-01-02T23:04:15.123430',  # Microseconds ends in zero
            '1971-01-02T23:04:15.0',  # Microseconds ends in zero

            # Should be valid but python datetime don't support it
            # '2005-12-31T23:59:60Z', # Leap second
            # '-0001', # BC 1
    ]:
        dt = ParseISO8601(testStr)
        if dt is None:
            print('Failed to parse ({0})'.format(testStr))
            assert (False)

        # Make sure we can translate back
        isoformat = ISO8601Format(dt)
        dt1 = ParseISO8601(isoformat)
        if dt.tzinfo is None:
            dt = dt.replace(tzinfo=dt1.tzinfo)
        if dt1 != dt:
            print('ParseISO8601 -> ISO8601Format -> ParseISO8601 failed (%s)' %
                  testStr)
            assert (False)

        # Make sure we can parse python isoformat()
        dt2 = ParseISO8601(dt.isoformat())
        if dt2 is None:
            print('ParseISO8601("{0}".isoformat()) failed'.format(testStr))
            assert (False)

        print(testStr, '->', dt, isoformat)

    # Basic form
    for testStr in [
            '197111',  # 1971-11-01
            '19711102',
            '19711102T23',
            '19711102T23Z',
            '19711102T2304',
            '19711102T2304Z',
            '19711102T230415',
            '19711102T230415Z',
            '19711102T230415.123456',
            '19711102T230415.123456Z',
            '19711102T230415.123456+11',
            '19711102T230415.123456-11',
            '19711102T230415.123456+1130',
            '19711102T230415.123456-1130',
    ]:
        # Reject for now
        dt = ParseISO8601(testStr)
        if dt is not None:
            print('ParseISO8601 ({0}) should fail, but it did not'.format(testStr))
            assert (False)
        # print testStr, '->', dt
        # assert(dt != None)

    # Invalid entries
    for testStr in [
            # Xml schema reject year 0
            '0000',  # 0 years are not allowed
            '+0001',  # Leading + is not allowed
            '',  # Empty datetime str
            '09',  # Years must be at least 4 digits
            '1971-01-02T',  # T not follow by time
            '1971-01-02TZ',  # T not follow by time
            '1971-01-02T+10',  # T not follow by time
            '1971-01-02T-10',  # T not follow by time
            '1971-01-02T23:',  # extra :
            '1971-01-02T23:04:',  # extra :
            '1971-01-02T23:0d',  # 0d
            '1971-01-02T23:04:15.',  # Dot not follows by microsec
            '1971-01-02+12',  # time without T
            '1971Z',  # Z without T
            '1971-01-02T23:04:15.123456Z+11',  # Z follows by +
            '1971-01-02T23:04:15.123456Z-11',  # Z follows by -
            '1971-01-02T23:04:15.123456+:30',  # extra :
            '1971-01-02T23:04:15.123456+30:',  # extra :
            # Too many microseconds digits
            '1971-01-02T23:04:15.01234567890123456789',

            # Python isoformat leave trailing zeros in microseconds
            # Relax regular expression to accept it
            # '1971-01-02T23:04:15.123430', # Microseconds ends in zero
            # '1971-01-02T23:04:15.0', # Microseconds ends in zero

            # Timezone must be between +14 / -14
            '1971-01-02T23:04:15+15',
            '1971-01-02T23:04:15-15',
            '1971-01-02T23:04:15+14:01',
            '1971-01-02T23:04:15-14:01',

            # Mix basic form with extended format
            '197101-02T23:04:15.123456',
            '19710102T23:04:15.123456',
            '19710102T230415.123456+11:30',
            '1971-01-02T230415.123456',
            '1971-01-02T23:04:15.123456+1130',

            # Error captured by datetime class
            '1971-00-02',  # Less than 1 month
            '1971-13-02',  # Larger than 12 months
            '1971-01-00',  # Less than 1 day
            '1971-11-32',  # Larger than 30 days for Nov
            '1971-12-32',  # Larger than 31 days
            '1971-01-02T24:01',  # Larger than 23 hr
            '1971-01-02T23:61',  # Larger than 60 min
            '1971-01-02T23:60:61',  # Larger than 61 sec
    ]:
        dt = ParseISO8601(testStr)
        if dt is not None:
            print('ParseISO8601 ({0}) should fail, but it did not'.format(testStr))
            assert (False)



from pyVmomi import vmodl
from pyVmomi.VmomiSupport import long


class DatastoreSpaceStatistics(vmodl.DynamicData):
    @property
    def profileId(self) -> str: ...
    @profileId.setter
    def profileId(self, value: str):
        self._profileId = value
    @property
    def physicalTotalInMB(self) -> long: ...
    @physicalTotalInMB.setter
    def physicalTotalInMB(self, value: long):
        self._physicalTotalInMB = value
    @property
    def physicalFreeInMB(self) -> long: ...
    @physicalFreeInMB.setter
    def physicalFreeInMB(self, value: long):
        self._physicalFreeInMB = value
    @property
    def physicalUsedInMB(self) -> long: ...
    @physicalUsedInMB.setter
    def physicalUsedInMB(self, value: long):
        self._physicalUsedInMB = value
    @property
    def logicalLimitInMB(self) -> long: ...
    @logicalLimitInMB.setter
    def logicalLimitInMB(self, value: long):
        self._logicalLimitInMB = value
    @property
    def logicalFreeInMB(self) -> long: ...
    @logicalFreeInMB.setter
    def logicalFreeInMB(self, value: long):
        self._logicalFreeInMB = value
    @property
    def logicalUsedInMB(self) -> long: ...
    @logicalUsedInMB.setter
    def logicalUsedInMB(self, value: long):
        self._logicalUsedInMB = value